#
# Brian C. Lane <bcl@redhat.com>
#
# Copyright 2014 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import unittest

from pykickstart.errors import KickstartParseWarning
from tests.baseclass import CommandTest, CommandSequenceTest
from pykickstart.commands.sshkey import F22_SshKeyData
from pykickstart.version import F22


class SshKey_TestCase(unittest.TestCase):
    def runTest(self):
        data1 = F22_SshKeyData()
        data2 = F22_SshKeyData()

        # test that new objects are always equal
        self.assertEqual(data1, data2)
        self.assertNotEqual(data1, None)

        # test for objects difference
        for atr in ['username']:
            setattr(data1, atr, '')
            setattr(data2, atr, 'test')
            # objects that differ in only one attribute
            # are not equal
            self.assertNotEqual(data1, data2)
            self.assertNotEqual(data2, data1)
            setattr(data1, atr, '')
            setattr(data2, atr, '')


class F22_TestCase(CommandTest):
    command = "sshkey"
    key = "ecdsa-sha2-nistp256 AAAAE2VjZHNhLXNoYTItbmlzdHAyNTYAAAAIbmlzdHAyNTYAAABBBJGDmFSzIWSvnFYhExf+FbzSiZxsoohJdrKlmPKQhdts8nSg5PH7jyG5X+w6RgWhSetlD3WouKoo3zFOR5nCYq4= bcl@notae.us"

    def runTest(self):
        # pass
        self.assert_parse('sshkey --username=root "%s"' % self.key, 'sshkey --username=root "%s"\n' % self.key)

        self.assertFalse(self.assert_parse("sshkey --username=root '%s'" % self.key) is None)
        self.assertTrue(self.assert_parse("sshkey --username=A '%s'" % self.key) !=
                        self.assert_parse("sshkey --username=B '%s'" % self.key))
        self.assertFalse(self.assert_parse("sshkey --username=A '%s'" % self.key) ==
                         self.assert_parse("sshkey --username=B '%s'" % self.key))

        # fail
        self.assert_parse_error("sshkey")
        self.assert_parse_error("sshkey --foo")
        self.assert_parse_error("sshkey --username=root --bogus-option")
        self.assert_parse_error("sshkey --username")
        self.assert_parse_error("sshkey --username=root")
        self.assert_parse_error("sshkey --username=root key1 key2")

        # extra test coverage
        sshkey = self.handler().commands[self.command]
        sshkey.sshUserList.append("someguy")
        self.assertEqual(sshkey.__str__(), "someguy")

class F22_Duplicate_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = F22

    def runTest(self):
        self.assert_parse("""
sshkey --username=someguy 'this is the key'
sshkey --username=otherguy 'this is the key'""")

        self.assert_parse_error("""
sshkey --username=someguy 'this is the key'
sshkey --username=someguy 'this is the key'""", KickstartParseWarning)

if __name__ == "__main__":
    unittest.main()



#
# James Laska <jlaska@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import unittest

from tests.baseclass import CommandTest
from pykickstart.base import DeprecatedCommand
from pykickstart.commands.monitor import FC6_Monitor

class FC3_TestCase(CommandTest):
    command = "monitor"

    def runTest(self):
        # pass
        self.assert_parse("monitor", "")
        self.assert_parse("monitor --hsync=HSYNC", "monitor --hsync=HSYNC\n")
        self.assert_parse("monitor --vsync=VSYNC", "monitor --vsync=VSYNC\n")
        self.assert_parse("monitor --monitor=MONITOR", "monitor --monitor=\"MONITOR\"\n")
        self.assert_parse("monitor --hsync=HSYNC --monitor=MONITOR",
                          "monitor --hsync=HSYNC --monitor=\"MONITOR\"\n")
        self.assert_parse("monitor --monitor=MONITOR --vsync=VSYNC",
                          "monitor --monitor=\"MONITOR\" --vsync=VSYNC\n")
        self.assert_parse("monitor --hsync=HSYNC --monitor=MONITOR --vsync=VSYNC",
                          "monitor --hsync=HSYNC --monitor=\"MONITOR\" --vsync=VSYNC\n")

        self.assert_parse_error("monitor BOGUS")
        self.assert_parse_error("monitor --monitor=SOMETHING GREAT")

        if "--noprobe" not in self.optionList:
            self.assert_parse_error("monitor --noprobe")

class FC6_TestCase(FC3_TestCase):
    def runTest(self):
        FC3_TestCase.runTest(self)

        # pass
        self.assert_parse("monitor --noprobe", "monitor --noprobe\n")
        # fail
        self.assert_parse_error("monitor --noprobe 1")
        # assert default values
        self.assertTrue(FC6_Monitor().probe)

class F10_TestCase(FC6_TestCase):
    def runTest(self):
        # make sure we've been deprecated
        parser = self.getParser("monitor")
        self.assertEqual(issubclass(parser.__class__, DeprecatedCommand), True)

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007, 2008, 2012 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.base import KickstartCommand
from pykickstart.version import versionToLongString, RHEL6, RHEL7, RHEL8
from pykickstart.version import FC3, F9, F12, F16, F17, F18, F20, F21, F26, F29
from pykickstart.constants import AUTOPART_TYPE_BTRFS, AUTOPART_TYPE_LVM, AUTOPART_TYPE_LVM_THINP, AUTOPART_TYPE_PLAIN
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class FC3_AutoPart(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.autopart = kwargs.get("autopart", False)

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.autopart:
            retval += "autopart\n"

        return retval

    def parse(self, args):
        if len(args) > 0:
            raise KickstartParseError(_("Kickstart command %s does not take any arguments") % "autopart", lineno=self.lineno)

        self.autopart = True
        return self

    def _getParser(self):
        return KSOptionParser(prog="autopart", description="""
                            Automatically create partitions -- a root (``/``) partition,
                            a swap partition, and an appropriate boot partition
                            for the architecture. On large enough drives, this
                            will also create a /home partition.

                            The ``autopart`` command can't be used with the logvol,
                            part/partition, raid, reqpart, or volgroup in the same
                            kickstart file.""", version=FC3)

class F9_AutoPart(FC3_AutoPart):
    removedKeywords = FC3_AutoPart.removedKeywords
    removedAttrs = FC3_AutoPart.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        FC3_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.encrypted = kwargs.get("encrypted", False)
        self.passphrase = kwargs.get("passphrase", "")

        self.op = self._getParser()

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if not self.autopart:
            return retval

        retval += "autopart"

        if self.encrypted:
            retval += " --encrypted"

            if self.passphrase:
                retval += " --passphrase=\"%s\""% self.passphrase

        retval += "\n"
        return retval

    def _getParser(self):
        op = FC3_AutoPart._getParser(self)
        op.add_argument("--encrypted", action="store_true", default=False,
                        version=F9, help="""
                        Should all devices with support be encrypted by default?
                        This is equivalent to checking the "Encrypt" checkbox on
                        the initial partitioning screen.""")
        op.add_argument("--passphrase", version=F9, help="""
                        Only relevant if ``--encrypted`` is specified. Provide
                        a default system-wide passphrase for all encrypted
                        devices.""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        self.autopart = True
        return self

class F12_AutoPart(F9_AutoPart):
    removedKeywords = F9_AutoPart.removedKeywords
    removedAttrs = F9_AutoPart.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        F9_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)

        self.escrowcert = kwargs.get("escrowcert", "")
        self.backuppassphrase = kwargs.get("backuppassphrase", False)

    def __str__(self):
        retval = F9_AutoPart.__str__(self)

        if not self.autopart:
            return retval

        if self.encrypted and self.escrowcert:
            retval = retval.strip()

            retval += " --escrowcert=\"%s\"" % self.escrowcert

            if self.backuppassphrase:
                retval += " --backuppassphrase"

            retval += "\n"

        return retval

    def _getParser(self):
        op = F9_AutoPart._getParser(self)
        op.add_argument("--escrowcert", metavar="<url>", version=F12, help="""
                        Only relevant if ``--encrypted`` is specified. Load an
                        X.509 certificate from ``<url>``. Store the data
                        encryption keys of all encrypted volumes created during
                        installation, encrypted using the certificate, as files
                        in ``/root``.""")
        op.add_argument("--backuppassphrase", action="store_true",
                        default=False, version=F12, help="""
                        Only relevant if ``--escrowcert`` is specified. In
                        addition to storing the data encryption keys, generate
                        a random passphrase and add it to all encrypted volumes
                        created during installation. Then store the passphrase,
                        encrypted using the certificate specified by
                        ``--escrowcert``, as files in ``/root`` (one file for
                        each encrypted volume).""")
        return op

class RHEL6_AutoPart(F12_AutoPart):
    removedKeywords = F12_AutoPart.removedKeywords
    removedAttrs = F12_AutoPart.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        F12_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.cipher = kwargs.get("cipher", "")

    def __str__(self):
        retval = F12_AutoPart.__str__(self)
        if not self.autopart:
            return retval

        if self.encrypted and self.cipher:
            # remove any trailing newline
            retval = retval.strip()
            retval += " --cipher=\"%s\"" % self.cipher
            retval += "\n"

        return retval

    def _getParser(self):
        op = F12_AutoPart._getParser(self)
        op.add_argument("--cipher", version=RHEL6, help="""
                        Only relevant if ``--encrypted`` is specified. Specifies
                        which encryption algorithm should be used to encrypt the
                        filesystem.""")
        return op

    def parse(self, args):
        # call the overriden command to do its job first
        retval = F12_AutoPart.parse(self, args)

        # Using autopart together with other partitioning command such as
        # part/partition, raid, logvol or volgroup can lead to hard to debug
        # behavior that might among other result into an unbootable system.
        #
        # Therefore if any of those commands is detected in the same kickstart
        # together with autopart, an error is raised and installation is
        # aborted.
        conflicting_command = ""

        # seen indicates that the corresponding
        # command has been seen in kickstart
        if self.handler.partition.seen:
            conflicting_command = "part/partition"
        elif self.handler.raid.seen:
            conflicting_command = "raid"
        elif self.handler.volgroup.seen:
            conflicting_command = "volgroup"
        elif self.handler.logvol.seen:
            conflicting_command = "logvol"

        if conflicting_command:
            # allow for translation of the error message
            errorMsg = _("The %s and autopart commands can't be used at the same time") % conflicting_command
            raise KickstartParseError(errorMsg, lineno=self.lineno)
        return retval


class F16_AutoPart(F12_AutoPart):
    removedKeywords = F12_AutoPart.removedKeywords
    removedAttrs = F12_AutoPart.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        F12_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.lvm = kwargs.get("lvm", True)

    def __str__(self):
        retval = F12_AutoPart.__str__(self)
        if not self.autopart:
            return retval

        # If requested, disable LVM autopart
        if not self.lvm:
            # remove any trailing newline
            retval = retval.strip()
            retval += " --nolvm"
            retval += "\n"

        return retval

    def _getParser(self):
        op = F12_AutoPart._getParser(self)
        op.add_argument("--nolvm", action="store_false", dest="lvm",
                        default=True, version=F16,
                        help="Don't use LVM when partitioning.")
        return op

class F17_AutoPart(F16_AutoPart):
    def __init__(self, writePriority=100, *args, **kwargs):
        self.typeMap = {"lvm": AUTOPART_TYPE_LVM,
                        "btrfs": AUTOPART_TYPE_BTRFS,
                        "plain": AUTOPART_TYPE_PLAIN,
                        "partition": AUTOPART_TYPE_PLAIN}
        F16_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.type = kwargs.get("type", None)

    def _typeAsStr(self):
        retval = None

        for (key, value) in list(self.typeMap.items()):
            if value == self.type:
                retval = key
                break

        if retval == "partition":
            retval = "plain"

        return retval

    def __str__(self):
        retval = F16_AutoPart.__str__(self)
        if not self.autopart:
            return retval

        ty = self._typeAsStr()
        if ty:
            # remove any trailing newline
            retval = retval.strip()
            retval += " --type=%s\n" % ty

        return retval

    def _type_cb(self, value):
        if value.lower() in self.typeMap:
            return self.typeMap[value.lower()]
        else:
            raise KickstartParseError(_("Invalid autopart type: %s") % value, lineno=self.lineno)

    def _getParser(self):
        op = F16_AutoPart._getParser(self)
        op.add_argument("--nolvm", action="store_const", version=F17,
                        const=AUTOPART_TYPE_PLAIN, dest="type",
                        help="The same as ``--type=plain``")
        op.add_argument("--type", type=self._type_cb, version=F17, help="""
                        Select automatic partitioning scheme. Must be one of the
                        following: %s. Plain means regular
                        partitions with no btrfs or lvm.""" % list(self.typeMap.keys()))
        return op

    def parse(self, args):
        retval = F16_AutoPart.parse(self, args)

        # make this always True to avoid writing --nolvm
        self.lvm = True

        return retval

class F18_AutoPart(F17_AutoPart):
    removedKeywords = F17_AutoPart.removedKeywords
    removedAttrs = F17_AutoPart.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        F17_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.cipher = kwargs.get("cipher", "")

    def __str__(self):
        retval = F17_AutoPart.__str__(self)
        if not self.autopart:
            return retval

        if self.encrypted and self.cipher:
            # remove any trailing newline
            retval = retval.strip()
            retval += " --cipher=\"%s\"" % self.cipher
            retval += "\n"

        return retval

    def _getParser(self):
        op = F17_AutoPart._getParser(self)
        op.add_argument("--cipher", version=F18, help="""
                        Only relevant if ``--encrypted`` is specified. Specifies
                        which encryption algorithm should be used to encrypt the
                        filesystem.""")
        return op

class F20_AutoPart(F18_AutoPart):
    def __init__(self, writePriority=100, *args, **kwargs):
        F18_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.typeMap["thinp"] = AUTOPART_TYPE_LVM_THINP

    def parse(self, args):
        # call the overriden command to do its job first
        retval = F18_AutoPart.parse(self, args)

        # Using autopart together with other partitioning command such as
        # part/partition, raid, logvol or volgroup can lead to hard to debug
        # behavior that might among other result into an unbootable system.
        #
        # Therefore if any of those commands is detected in the same kickstart
        # together with autopart, an error is raised and installation is
        # aborted.
        conflicting_command = ""

        # seen indicates that the corresponding
        # command has been seen in kickstart
        if self.handler.partition.seen:
            conflicting_command = "part/partition"
        elif self.handler.raid.seen:
            conflicting_command = "raid"
        elif self.handler.volgroup.seen:
            conflicting_command = "volgroup"
        elif self.handler.logvol.seen:
            conflicting_command = "logvol"
        elif hasattr(self.handler, "mount") and self.handler.mount.seen:
            conflicting_command = "mount"

        if conflicting_command:
            # allow for translation of the error message
            errorMsg = _("The %s and autopart commands can't be used at the same time") % conflicting_command
            raise KickstartParseError(errorMsg, lineno=self.lineno)
        return retval

    def _getParser(self):
        "Only necessary for the type change documentation"
        op = F18_AutoPart._getParser(self)
        for action in op._actions:
            if "--type" in action.option_strings:
                action.help += """

                    .. versionchanged:: %s

                    Partitioning scheme 'thinp' was added.""" % versionToLongString(F20)
        return op

class F21_AutoPart(F20_AutoPart):
    removedKeywords = F20_AutoPart.removedKeywords
    removedAttrs = F20_AutoPart.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        F20_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.fstype = kwargs.get("fstype", "")

    def __str__(self):
        retval = F20_AutoPart.__str__(self)
        if not self.autopart:
            return retval

        if self.fstype:
            # remove any trailing newline
            retval = retval.strip()
            retval += " --fstype=%s" % self.fstype
            retval += "\n"

        return retval

    def _getParser(self):
        op = F20_AutoPart._getParser(self)
        op.add_argument("--fstype", version=F21, help="""
                        Use the specified filesystem type on the partitions.
                        Note that it cannot be used with ``--type=btrfs`` since
                        btrfs is both a partition scheme and a filesystem. eg.
                        ``--fstype=ext4``.""")
        return op

    def parse(self, args):
        # call the overriden command to do its job first
        retval = F20_AutoPart.parse(self, args)

        # btrfs is not a valid filesystem type
        if self.fstype == "btrfs":
            raise KickstartParseError(_("autopart --fstype=btrfs is not valid fstype, use --type=btrfs instead"), lineno=self.lineno)

        if self._typeAsStr() == "btrfs" and self.fstype:
            raise KickstartParseError(_("autopart --fstype cannot be used with --type=btrfs"), lineno=self.lineno)

        return retval

class F23_AutoPart(F21_AutoPart):
    def parse(self, args):
        # call the overriden command to do its job first
        retval = F21_AutoPart.parse(self, args)

        conflicting_command = ""
        if hasattr(self.handler, "reqpart") and self.handler.reqpart.seen:
            conflicting_command = "reqpart"

        if conflicting_command:
            # allow for translation of the error message
            errorMsg = _("The %s and autopart commands can't be used at the same time") % conflicting_command
            raise KickstartParseError(errorMsg, lineno=self.lineno)

        return retval

class RHEL7_AutoPart(F21_AutoPart):

    def __init__(self, writePriority=100, *args, **kwargs):
        F21_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.nohome = kwargs.get("nohome", False)

    def __str__(self):
        retval = F21_AutoPart.__str__(self)
        if not self.autopart:
            return retval

        if self.nohome:
            # remove any trailing newline
            retval = retval.strip()
            retval += " --nohome"
            retval += "\n"

        return retval

    def _getParser(self):
        op = F21_AutoPart._getParser(self)
        op.add_argument("--nohome", action="store_true", default=False,
                        version=RHEL7, help="""
                        Do not create a /home partition.""")
        return op

    def parse(self, args):
        # call the overriden command to do its job first
        retval = F21_AutoPart.parse(self, args)

        conflicting_command = ""
        if hasattr(self.handler, "reqpart") and self.handler.reqpart.seen:
            conflicting_command = "reqpart"

        if conflicting_command:
            # allow for translation of the error message
            errorMsg = _("The %s and autopart commands can't be used at the same time") % conflicting_command
            raise KickstartParseError(errorMsg, lineno=self.lineno)

        return retval

class F26_AutoPart(F23_AutoPart):
    removedKeywords = F23_AutoPart.removedKeywords
    removedAttrs = F23_AutoPart.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        F23_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.nohome = kwargs.get("nohome", False)
        self.noboot = kwargs.get("noboot", False)
        self.noswap = kwargs.get("noswap", False)

    def __str__(self):
        retval = F23_AutoPart.__str__(self)
        if not self.autopart:
            return retval

        if self.nohome:
            # remove any trailing newline
            retval = retval.strip()
            retval += " --nohome"
            retval += "\n"

        if self.noboot:
            # remove any trailing newline
            retval = retval.strip()
            retval += " --noboot"
            retval += "\n"

        if self.noswap:
            # remove any trailing newline
            retval = retval.strip()
            retval += " --noswap"
            retval += "\n"

        return retval

    def _getParser(self):
        op = F23_AutoPart._getParser(self)
        op.add_argument("--nohome", action="store_true", default=False,
                        version=F26, help="""
                        Do not create a /home partition.""")
        op.add_argument("--noboot", action="store_true", default=False,
                        version=F26, help="""
                        Do not create a /boot partition.""")
        op.add_argument("--noswap", action="store_true", default=False,
                        version=F26, help="""
                        Do not create a swap partition.""")
        return op

class F29_AutoPart(F26_AutoPart):
    removedKeywords = F26_AutoPart.removedKeywords
    removedAttrs = F26_AutoPart.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        F26_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.luks_version = kwargs.get("luks_version", "")
        self.pbkdf = kwargs.get("pbkdf", "")
        self.pbkdf_memory = kwargs.get("pbkdf_memory", 0)
        self.pbkdf_time = kwargs.get("pbkdf_time", 0)
        self.pbkdf_iterations = kwargs.get("pbkdf_iterations", 0)

    def __str__(self):
        retval = F26_AutoPart.__str__(self)
        if not self.autopart:
            return retval

        if self.encrypted and self.luks_version:
            retval = retval.strip()
            retval += " --luks-version=%s" % self.luks_version
            retval += "\n"

        if self.encrypted and self.pbkdf:
            retval = retval.strip()
            retval += " --pbkdf=%s" % self.pbkdf
            retval += "\n"

        if self.encrypted and self.pbkdf_memory:
            retval = retval.strip()
            retval += " --pbkdf-memory=%s" % self.pbkdf_memory
            retval += "\n"

        if self.encrypted and self.pbkdf_time:
            retval = retval.strip()
            retval += " --pbkdf-time=%s" % self.pbkdf_time
            retval += "\n"

        if self.encrypted and self.pbkdf_iterations:
            retval = retval.strip()
            retval += " --pbkdf-iterations=%s" % self.pbkdf_iterations
            retval += "\n"

        return retval

    def _getParser(self):
        op = F26_AutoPart._getParser(self)
        op.add_argument("--luks-version", dest="luks_version", version=F29, default="",
                        help="""
                        Only relevant if ``--encrypted`` is specified. Specifies
                        which version of LUKS format should be used to encrypt
                        the filesystem.""")
        op.add_argument("--pbkdf", version=F29, default="", help="""
                        Only relevant if ``--encrypted`` is specified. Sets
                        Password-Based Key Derivation Function (PBKDF) algorithm
                        for LUKS keyslot. See ``man cryptsetup``.""")
        op.add_argument("--pbkdf-memory", dest="pbkdf_memory", type=int, default=0,
                        version=F29, help="""
                        Only relevant if ``--encrypted`` is specified. Sets
                        the memory cost for PBKDF. See ``man cryptsetup``.""")
        op.add_argument("--pbkdf-time", dest="pbkdf_time", type=int, default=0,
                        version=F29, help="""
                        Only relevant if ``--encrypted`` is specified. Sets
                        the number of milliseconds to spend with PBKDF passphrase
                        processing. See ``--iter-time`` in ``man cryptsetup``.

                        Only one of ``--pbkdf-time`` and ``--pbkdf-iterations``
                        can be specified.
                        """)
        op.add_argument("--pbkdf-iterations", dest="pbkdf_iterations", type=int, default=0,
                        version=F29, help="""
                        Only relevant if ``--encrypted`` is specified. Sets
                        the number of iterations directly and avoids PBKDF benchmark.
                        See ``--pbkdf-force-iterations`` in ``man cryptsetup``.

                        Only one of ``--pbkdf-time`` and ``--pbkdf-iterations``
                        can be specified.
                        """)
        return op

    def parse(self, args):
        retval = F26_AutoPart.parse(self, args)

        if self.pbkdf_time and self.pbkdf_iterations:
            msg = _("Only one of --pbkdf-time and --pbkdf-iterations can be specified.")
            raise KickstartParseError(msg, lineno=self.lineno)

        return retval

class RHEL8_AutoPart(F29_AutoPart):
    removedKeywords = F29_AutoPart.removedKeywords
    removedAttrs = F29_AutoPart.removedAttrs

    def parse(self, args):
        # call the overriden command to do it's job first
        retval = F29_AutoPart.parse(self, args)

        # btrfs is no more supported
        if self._typeAsStr() == "btrfs":
            raise KickstartParseError(_("autopart --type=btrfs is not supported"),
                                      lineno=self.lineno)

        return retval

    def _getParser(self):
        "Only necessary for the type change documentation"
        op = F29_AutoPart._getParser(self)
        for action in op._actions:
            if "--type" in action.option_strings:
                action.help += """

                    .. versionchanged:: %s

                    Partitioning scheme 'btrfs' was removed.""" % versionToLongString(RHEL8)
            if "--fstype" in action.option_strings:
                action.help += """

                    .. versionchanged:: %s

                    Partitioning scheme 'btrfs' was removed.""" % versionToLongString(RHEL8)
        return op



#
# Paul W. Frields <pfrields@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import unittest
from tests.baseclass import CommandTest

class FC3_TestCase(CommandTest):
    command = "keyboard"

    def runTest(self):
        # pass
        self.assert_parse("keyboard us", "keyboard us\n")

        # fail
        self.assert_parse_error("keyboard")
        self.assert_parse_error("keyboard us uk")
        self.assert_parse_error("keyboard --foo us")
        self.assert_parse_error("keyboard --bogus-option")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.keyboard = ""
        self.assertEqual(cmd.__str__(), "")


class F18_TestCase(FC3_TestCase):
    def runTest(self):
        # pass
        self.assert_parse("keyboard us",
                          "keyboard 'us'\n")
        self.assert_parse("keyboard --vckeymap=us",
                          "keyboard --vckeymap=us\n")
        # This is tedious - I'm only going to check it once.
        self.assert_parse("keyboard --vckeymap=us sk",
                          "# old format: keyboard sk\n# new format:\nkeyboard --vckeymap=us\n")
        self.assert_parse("keyboard --xlayouts='cz (qwerty)'",
                          "keyboard --xlayouts='cz (qwerty)'\n")
        self.assert_parse("keyboard --xlayouts=cz,'cz (qwerty)'",
                          "keyboard --xlayouts='cz','cz (qwerty)'\n")
        self.assert_parse("keyboard --xlayouts=cz sk")
        self.assert_parse("keyboard --vckeymap=us --xlayouts=cz",
                          "keyboard --vckeymap=us --xlayouts='cz'\n")
        self.assert_parse("keyboard --vckeymap=us --xlayouts=cz,'cz (qwerty)' sk")
        self.assert_parse("keyboard --vckeymap=us --xlayouts=cz --switch=grp:alt_shift_toggle",
                          "keyboard --vckeymap=us --xlayouts='cz' --switch='grp:alt_shift_toggle'\n")
        self.assert_parse("keyboard --vckeymap=us --xlayouts=cz --switch=grp:alt_shift_toggle,grp:switch",
                          "keyboard --vckeymap=us --xlayouts='cz' --switch='grp:alt_shift_toggle','grp:switch'\n")

        # fail
        self.assert_parse_error("keyboard")
        self.assert_parse_error("keyboard cz sk")
        self.assert_parse_error("keyboard --vckeymap=us --xlayouts=cz,"
                                "'cz (qwerty)' cz sk")
        self.assert_parse_error("keyboard --foo us")
        self.assert_parse_error("keyboard --bogus-option")

        # keyboard property
        obj = self.assert_parse("keyboard us")
        self.assertEqual(obj.keyboard, "us")

        obj = self.assert_parse("keyboard --xlayouts=us,cz")
        self.assertEqual(obj.keyboard, "us")

        obj = self.assert_parse("keyboard --xlayouts=,bg")
        self.assertEqual(obj.x_layouts, ["bg"])

        obj.keyboard = "cz"
        self.assertEqual(obj.keyboard, "cz")

if __name__ == "__main__":
    unittest.main()



import unittest
from tests.baseclass import CommandTest

class FC3_TestCase(CommandTest):
    command = "selinux"

    def runTest(self):
        # pass
        self.assert_parse("selinux")
        self.assert_parse("selinux --permissive", "selinux --permissive\n")
        self.assert_parse("selinux --enforcing", "selinux --enforcing\n")
        self.assert_parse("selinux --disabled", "selinux --disabled\n")

        # fail
        self.assert_parse_error("selinux --cheese")
        self.assert_parse_error("selinux --crackers=CRUNCHY")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.selinux = 999
        self.assertEqual(cmd.__str__(), "# SELinux configuration\n")


if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005-2016 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
CLEARPART_TYPE_LINUX = 0
CLEARPART_TYPE_ALL = 1
CLEARPART_TYPE_NONE = 2
CLEARPART_TYPE_LIST = 3

AUTOPART_TYPE_PLAIN = 0
AUTOPART_TYPE_BTRFS = 1
AUTOPART_TYPE_LVM = 2
AUTOPART_TYPE_LVM_THINP = 3

DISPLAY_MODE_CMDLINE = 0
DISPLAY_MODE_GRAPHICAL = 1
DISPLAY_MODE_TEXT = 2

FIRSTBOOT_DEFAULT = 0
FIRSTBOOT_SKIP = 1
FIRSTBOOT_RECONFIG = 2

KS_MISSING_PROMPT = 0
KS_MISSING_IGNORE = 1

SELINUX_DISABLED = 0
SELINUX_ENFORCING = 1
SELINUX_PERMISSIVE = 2

SNAPSHOT_WHEN_POST_INSTALL = 0
SNAPSHOT_WHEN_PRE_INSTALL = 1

KS_SCRIPT_PRE = 0
KS_SCRIPT_POST = 1
KS_SCRIPT_TRACEBACK = 2
KS_SCRIPT_PREINSTALL = 3
KS_SCRIPT_ONERROR = 4

KS_WAIT = 0
KS_REBOOT = 1
KS_SHUTDOWN = 2

KS_INSTKEY_SKIP = -99

BOOTPROTO_DHCP = "dhcp"
BOOTPROTO_BOOTP = "bootp"
BOOTPROTO_STATIC = "static"
BOOTPROTO_QUERY = "query"
BOOTPROTO_IBFT = "ibft"

GROUP_REQUIRED = 0
GROUP_DEFAULT = 1
GROUP_ALL = 2

BIND_TO_MAC = "mac"

NVDIMM_MODE_SECTOR = "sector"

NVDIMM_ACTION_RECONFIGURE = "reconfigure"
NVDIMM_ACTION_USE = "use"

SECURE_BOOT_DEFAULT = -1
SECURE_BOOT_DISABLED = 0
SECURE_BOOT_ENABLED = 1
SECURE_BOOT_AUTO = 2



# -*- coding: utf-8 -*-
import os
import six
import unittest
import tempfile
import locale

from tests.baseclass import ParserTest

class HandleUnicode_TestCase(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
rootpw ááááááááá

%post
echo áááááá
%end
"""

    def get_encoded_str(self, string, force_encode=False):
        if force_encode or not six.PY3:
            return string.encode("utf-8")
        else:
            return string

    def runTest(self):
        unicode_str1 = u"ááááááááá"
        unicode_str2 = u"áááááá"

        # pylint: disable=environment-modify
        # Make sure the locale is reset so that the traceback could happen
        del os.environ["LANG"]
        locale.resetlocale()

        # parser should parse string including non-ascii characters
        self.parser.readKickstartFromString(self.ks)

        # str(handler) should not cause traceback and should contain the
        # original non-ascii strings as utf-8 encoded byte strings and
        # str(self.handler) should not fail -- i.e. self.handler.__str__()
        # should return byte string not unicode string
        self.assertIn(self.get_encoded_str(unicode_str1), str(self.handler))
        self.assertIn(self.get_encoded_str(unicode_str2), str(self.handler))

        # set root password to unicode string
        self.handler.rootpw.password = unicode_str1

        # str(handler) should not cause traceback and should contain the
        # original unicode string as utf-8 encoded byte string
        self.assertIn(self.get_encoded_str(unicode_str1), str(self.handler))

        # set root password to encoded string
        self.handler.rootpw.password = self.get_encoded_str(unicode_str1, force_encode=True)

        # str(handler) should not cause traceback and should contain the
        # original unicode string as utf-8 encoded byte string
        self.assertIn(str(self.get_encoded_str(unicode_str1, force_encode=True)), str(self.handler))

        (fd, name) = tempfile.mkstemp(prefix="ks-", suffix=".cfg", dir="/tmp", text=True)
        if six.PY3:
            buf = self.ks.encode("utf-8")
        else:
            buf = self.ks
        os.write(fd, buf)
        os.close(fd)

        # This should not traceback with a UnicodeError
        try:
            self.parser.readKickstart(name)

            # str(handler) should not cause traceback and should contain the
            # original non-ascii strings as utf-8 encoded byte strings and
            # str(self.handler) should not fail -- i.e. self.handler.__str__()
            # should return byte string not unicode string
            self.assertIn(self.get_encoded_str(unicode_str1), str(self.handler))
            self.assertIn(self.get_encoded_str(unicode_str2), str(self.handler))
        finally:
            os.unlink(name)

if __name__ == "__main__":
    if not six.PY3:
        unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007, 2008 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, FC4, F12, F14
from pykickstart.base import BaseData, KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class FC3_DriverDiskData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)

        self.partition = kwargs.get("partition", "")
        self.source = kwargs.get("source", "")
        self.type = kwargs.get("type", "")

    def _getArgsAsStr(self):
        retval = ""

        if self.partition:
            retval += "%s" % self.partition

            if hasattr(self, "type") and self.type:
                retval += " --type=%s" % self.type
        elif self.source:
            retval += "--source=%s" % self.source
        return retval

    def __str__(self):
        retval = BaseData.__str__(self)
        retval += "driverdisk %s\n" % self._getArgsAsStr()
        return retval

class FC4_DriverDiskData(FC3_DriverDiskData):
    removedKeywords = FC3_DriverDiskData.removedKeywords
    removedAttrs = FC3_DriverDiskData.removedAttrs

    def __init__(self, *args, **kwargs):
        FC3_DriverDiskData.__init__(self, *args, **kwargs)
        self.deleteRemovedAttrs()

        self.biospart = kwargs.get("biospart", "")

    def _getArgsAsStr(self):
        retval = ""

        if self.partition:
            retval += "%s" % self.partition

            if hasattr(self, "type") and self.type:
                retval += " --type=%s" % self.type
        elif self.source:
            retval += "--source=%s" % self.source
        elif self.biospart:
            retval += "--biospart=%s" % self.biospart

        return retval

class F12_DriverDiskData(FC4_DriverDiskData):
    removedKeywords = FC4_DriverDiskData.removedKeywords + ["type"]
    removedAttrs = FC4_DriverDiskData.removedAttrs + ["type"]

    def __init__(self, *args, **kwargs):
        FC4_DriverDiskData.__init__(self, *args, **kwargs)
        self.deleteRemovedAttrs()

class F14_DriverDiskData(F12_DriverDiskData):
    pass

class FC3_DriverDisk(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.driverdiskList = kwargs.get("driverdiskList", [])

    def __str__(self):
        retval = ""
        for dd in self.driverdiskList:
            retval += dd.__str__()

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="driverdisk", description="""
                            Driver diskettes can be used during kickstart
                            installations. You need to copy the driver disk's
                            contents to the root directory of a partition on
                            the system's hard drive. Then you need to use the
                            driverdisk command to tell the installation program
                            where to look for the driver disk.""",
                            version=FC3)
        op.add_argument("partition", nargs="*", version=FC3,
                        help="Partition containing the driver disk.")
        op.add_argument("--source", version=FC3, help="""
                        Specify a URL for the driver disk. NFS locations can be
                        given with ``nfs:host:/path/to/img``.""")
        op.add_argument("--type", version=FC3, help="")
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if len(ns.partition) > 1:
            raise KickstartParseError(_("Only one partition may be specified for driverdisk command."), lineno=self.lineno)
        elif extra:
            mapping = {"command": "driverdisk", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        if len(ns.partition) == 1 and ns.source:
            raise KickstartParseError(_("Only one of --source and partition may be specified for driverdisk command."), lineno=self.lineno)

        if not ns.partition and not ns.source:
            raise KickstartParseError(_("One of --source or partition must be specified for driverdisk command."), lineno=self.lineno)

        ddd = self.dataClass()  # pylint: disable=not-callable
        self.set_to_obj(ns, ddd)
        ddd.lineno = self.lineno
        if len(ns.partition) == 1:
            ddd.partition = ns.partition[0]

        return ddd

    def dataList(self):
        return self.driverdiskList

    @property
    def dataClass(self):
        return self.handler.DriverDiskData

class FC4_DriverDisk(FC3_DriverDisk):
    removedKeywords = FC3_DriverDisk.removedKeywords
    removedAttrs = FC3_DriverDisk.removedKeywords

    def _getParser(self):
        op = FC3_DriverDisk._getParser(self)
        op.add_argument("--biospart", version=FC4, help="""
                        BIOS partition containing the driver disk (such as 82p2).
                        """)
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if len(ns.partition) > 1:
            raise KickstartParseError(_("Only one partition may be specified for driverdisk command."), lineno=self.lineno)
        elif extra:
            mapping = {"command": "driverdisk", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        if len(ns.partition) == 1 and ns.source:
            raise KickstartParseError(_("Only one of --source and partition may be specified for driverdisk command."), lineno=self.lineno)
        elif len(ns.partition) == 1 and ns.biospart:
            raise KickstartParseError(_("Only one of --biospart and partition may be specified for driverdisk command."), lineno=self.lineno)
        elif ns.source and ns.biospart:
            raise KickstartParseError(_("Only one of --biospart and --source may be specified for driverdisk command."), lineno=self.lineno)

        if not ns.partition and not ns.source and not ns.biospart:
            raise KickstartParseError(_("One of --source, --biospart, or partition must be specified for driverdisk command."), lineno=self.lineno)

        ddd = self.dataClass()  # pylint: disable=not-callable
        self.set_to_obj(ns, ddd)
        ddd.lineno = self.lineno
        if len(ns.partition) == 1:
            ddd.partition = ns.partition[0]

        return ddd

class F12_DriverDisk(FC4_DriverDisk):
    removedKeywords = FC4_DriverDisk.removedKeywords
    removedAttrs = FC4_DriverDisk.removedKeywords

    def _getParser(self):
        op = FC4_DriverDisk._getParser(self)
        op.add_argument("--type", deprecated=F12)
        return op

class F14_DriverDisk(F12_DriverDisk):
    removedKeywords = F12_DriverDisk.removedKeywords
    removedAttrs = F12_DriverDisk.removedKeywords

    def _getParser(self):
        op = F12_DriverDisk._getParser(self)
        op.remove_argument("--type", version=F14)
        return op



import unittest
from tests.baseclass import CommandTest, CommandSequenceTest
from pykickstart.commands.volgroup import FC3_VolGroupData, F21_VolGroupData
from pykickstart.errors import KickstartParseError, KickstartParseWarning
from pykickstart.version import FC3

class VolGroup_TestCase(unittest.TestCase):
    def runTest(self):
        data1 = FC3_VolGroupData()
        data2 = FC3_VolGroupData()

        # test default object values
        self.assertEqual(data1.format, True)
        self.assertEqual(data1.pesize, 32768)
        self.assertEqual(data1.preexist, False)

        self.assertEqual(F21_VolGroupData().pesize, 0)

        # test that new objects are always equal
        self.assertEqual(data1, data2)
        self.assertNotEqual(data1, None)

        # test for objects difference
        for atr in ['vgname']:
            setattr(data1, atr, '')
            setattr(data2, atr, 'test')
            # objects that differ in only one attribute
            # are not equal
            self.assertNotEqual(data1, data2)
            self.assertNotEqual(data2, data1)
            setattr(data1, atr, '')
            setattr(data2, atr, '')


class FC3_TestCase(CommandTest):
    command = "volgroup"

    def runTest(self):
        if self.__class__ in (FC3_TestCase, F16_TestCase, RHEL6_TestCase):
            def_pesize_str = " --pesize=32768"
        else:
            def_pesize_str = ""

        # --noformat
        self.assert_parse("volgroup vg.01 --noformat",
                          "volgroup vg.01 --noformat%s --useexisting\n" % def_pesize_str)
        # --useexisting
        self.assert_parse("volgroup vg.01 --useexisting",
                          "volgroup vg.01%s --useexisting\n" % def_pesize_str)

        # --pesize
        self.assert_parse("volgroup vg.01 pv.01 --pesize=70000",
                          "volgroup vg.01 --pesize=70000 pv.01\n")

        # assert data types
        self.assert_type("volgroup", "pesize", "int")
        self.assert_type("volgroup", "format", "boolean")
        self.assert_type("volgroup", "preexist", "boolean")

        self.assertFalse(self.assert_parse("volgroup vg.01 pv.01") is None)
        self.assertTrue(self.assert_parse("volgroup vg.01 pv.01") !=
                        self.assert_parse("volgroup vg.02 pv.01"))
        self.assertFalse(self.assert_parse("volgroup vg.01 pv.01") ==
                         self.assert_parse("volgroup vg.02 pv.01"))

        # fail - incorrect type
        self.assert_parse_error("volgroup vg.01 pv.01 --pesize=SIZE")

        # fail - missing name
        self.assert_parse_error("volgroup")

        # fail - missing list of partitions
        self.assert_parse_error("volgroup vg01")

        # fail - both members and useexisting specified
        self.assert_parse_error("volgroup vg.01 pv.01 pv.02 --useexisting")
        self.assert_parse_error("volgroup vg.01 --useexisting pv.01 pv.02")

        # fail - invalid argument
        self.assert_parse_error("volgroup --bogus-option")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.vgList = ["vg.01"]
        self.assertEqual(cmd.__str__(), "vg.01")

class FC3_Duplicate_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = FC3

    def runTest(self):
        self.assert_parse("""
volgroup vg.01 pv.01
volgroup vg.02 pv.01""")

        self.assert_parse_error("""
volgroup vg.01 pv.01
volgroup vg.01 pv.02""", KickstartParseWarning)

class F16_TestCase(FC3_TestCase):
    def runTest(self):
        FC3_TestCase.runTest(self)

        if self.__class__ in (FC3_TestCase, F16_TestCase, RHEL6_TestCase):
            def_pesize_str = " --pesize=32768"
        else:
            def_pesize_str = ""

        # Pass - correct usage.
        self.assert_parse("volgroup vg.01 pv.01 --reserved-space=1000",
                          "volgroup vg.01%s --reserved-space=1000 pv.01\n" % def_pesize_str)
        self.assert_parse("volgroup vg.01 pv.01 --reserved-percent=50",
                          "volgroup vg.01%s --reserved-percent=50 pv.01\n" % def_pesize_str)

        # Fail - missing required argument.
        self.assert_parse_error("volgroup vg.01 pv.01 --reserved-space")
        self.assert_parse_error("volgroup vg.01 pv.01 --reserved-percent")

        # Fail - incorrect values.
        self.assert_parse_error("volgroup vg.01 pv.01 --reserved-space=-1")
        self.assert_parse_error("volgroup vg.01 pv.01 --reserved-percent=0")
        self.assert_parse_error("volgroup vg.01 pv.01 --reserved-percent=100")

class RHEL6_TestCase(F16_TestCase):
    def runTest(self):
        F16_TestCase.runTest(self)
        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.parse(["volgroup", "vg.02", "pv.01"])
        self.assertEqual(cmd.__str__(), "")
        cmd.handler.autopart.seen = True
        with self.assertRaises(KickstartParseError):
            cmd.parse(["volgroup", "vg.02", "pv.01"])
        cmd.handler.autopart.seen = False

class F21_TestCase(F16_TestCase):
    def runTest(self):
        # just run all the old tests with the new class (different PE size
        # default)
        F16_TestCase.runTest(self)

class RHEL7_TestCase(F16_TestCase):
    def runTest(self):
        F16_TestCase.runTest(self)

if __name__ == "__main__":
    unittest.main()



#
# Vendula Poncova <vponcova@redhat.com>
#
# Copyright 2017 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest


class RHEL7_TestCase(CommandTest):
    def runTest(self):
        # pass
        self.assert_parse("hmc", "hmc\n")

        # fail
        self.assert_parse_error("hmc --some-arg")
        self.assert_parse_error("hmc somextracrud")

class F28_TestCase(RHEL7_TestCase):
    def runTest(self):
        RHEL7_TestCase.runTest(self)

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.errors import KickstartParseWarning
from pykickstart.version import F12
from pykickstart.base import BaseData, KickstartCommand
from pykickstart.options import KSOptionParser

import warnings
from pykickstart.i18n import _

class F12_GroupData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.name = kwargs.get("name", "")
        self.gid = kwargs.get("gid", None)

    def __eq__(self, y):
        if not y:
            return False

        return self.name == y.name

    def __ne__(self, y):
        return not self == y

    def __str__(self):
        retval = BaseData.__str__(self)
        retval += "group"

        if self.name:
            retval += " --name=%s" % self.name
        if self.gid:
            retval += " --gid=%s" % self.gid

        return retval + "\n"

class F12_Group(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.groupList = kwargs.get("groupList", [])

    def __str__(self):
        retval = ""
        for user in self.groupList:
            retval += user.__str__()

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="group", description="""
            Creates a new user group on the system. If a group with the given
            name or GID already exists, this command will fail. In addition,
            the ``user`` command can be used to create a new group for the
            newly created user.""", version=F12)
        op.add_argument("--name", required=True, version=F12,
                        help="Provides the name of the new group.")
        op.add_argument("--gid", type=int, version=F12, help="""
                        The group's GID. If not provided, this defaults to the
                        next available non-system GID.""")
        return op

    def parse(self, args):
        gd = self.dataClass()   # pylint: disable=not-callable
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_obj(ns, gd)
        gd.lineno = self.lineno

        # Check for duplicates in the data list.
        if gd in self.dataList():
            warnings.warn(_("A group with the name %s has already been defined.") % gd.name, KickstartParseWarning)

        return gd

    def dataList(self):
        return self.groupList

    @property
    def dataClass(self):
        return self.handler.GroupData



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007, 2012 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, F17, F21, F28
from pykickstart.base import KickstartCommand
from pykickstart.constants import CLEARPART_TYPE_ALL, CLEARPART_TYPE_LINUX, CLEARPART_TYPE_LIST, CLEARPART_TYPE_NONE
from pykickstart.options import KSOptionParser, commaSplit

class FC3_ClearPart(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=120, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.drives = kwargs.get("drives", [])
        self.initAll = kwargs.get("initAll", False)
        self.type = kwargs.get("type", None)

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.type is None:
            return retval

        if self.type == CLEARPART_TYPE_NONE:
            clearstr = " --none"
        elif self.type == CLEARPART_TYPE_LINUX:
            clearstr = " --linux"
        elif self.type == CLEARPART_TYPE_ALL:
            clearstr = " --all"
        else:
            clearstr = ""

        if self.initAll:
            initstr = " --initlabel"
        else:
            initstr = ""

        if self.drives:
            drivestr = " --drives=" + ",".join(self.drives)
        else:
            drivestr = ""

        retval += "# Partition clearing information\nclearpart%s%s%s\n" % (clearstr, initstr, drivestr)
        return retval

    def _getParser(self):
        op = KSOptionParser(prog="clearpart", description="""
                            Removes partitions from the system, prior to creation
                            of new partitions. By default, no partitions are
                            removed.

                            If the clearpart command is used, then the ``--onpart``
                            command cannot be used on a logical partition.""",
                            version=FC3)
        op.add_argument("--all", dest="type", action="store_const",
                        const=CLEARPART_TYPE_ALL, version=FC3,
                        help="Erases all partitions from the system.")
        op.add_argument("--drives", type=commaSplit, help="""
                        Specifies which drives to clear partitions from. For
                        example, the following clears the partitions on the
                        first two drives on the primary IDE controller::

                        ``clearpart --all --drives=sda,sdb``""",
                        version=FC3)
        op.add_argument("--initlabel", dest="initAll", action="store_true",
                        default=False, version=FC3, help="""
                        Initializes the disk label to the default for your
                        architecture (for example msdos for x86 and gpt for
                        Itanium). This is only meaningful in combination with
                        the '--all' option.""")
        op.add_argument("--linux", dest="type", action="store_const",
                        const=CLEARPART_TYPE_LINUX, version=FC3,
                        help="Erases all Linux partitions.")
        op.add_argument("--none", dest="type", action="store_const",
                        const=CLEARPART_TYPE_NONE, version=FC3, help="""
                        Do not remove any partitions. This is the default""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        return self

class F17_ClearPart(FC3_ClearPart):
    def __init__(self, *args, **kwargs):
        super(F17_ClearPart, self).__init__(*args, **kwargs)
        self.devices = kwargs.get("devices", [])

    def __str__(self):
        s = super(F17_ClearPart, self).__str__()
        if s and self.devices:
            s = s.rstrip()
            s += " --list=" + ",".join(self.devices)
            s += "\n"
        return s

    def _getParser(self):
        op = FC3_ClearPart._getParser(self)
        op.add_argument("--list", dest="devices", type=commaSplit,
                        version=F17, help="""
                        Specifies which partitions to clear. If given, this
                        supersedes any of the ``--all`` and ``--linux``
                        options. This can be across different drives::

                        ``clearpart --list=sda2,sda3,sdb1``""")
        return op

    def parse(self, args):
        obj = FC3_ClearPart.parse(self, args)
        if getattr(obj, "devices", []):
            obj.type = CLEARPART_TYPE_LIST

        return obj

class F21_ClearPart(F17_ClearPart):
    def __init__(self, *args, **kwargs):
        super(F21_ClearPart, self).__init__(*args, **kwargs)
        self.disklabel = kwargs.get("disklabel", "")

    def __str__(self):
        s = super(F21_ClearPart, self).__str__()
        if s and self.disklabel:
            s = s.rstrip()
            s += " --disklabel=%s\n" % self.disklabel
        return s

    def _getParser(self):
        op = F17_ClearPart._getParser(self)
        op.add_argument("--disklabel", default="", version=F21, help="""
                        Set the default disklabel to use. Only disklabels
                        supported for the platform will be accepted. eg. msdos
                        and gpt for x86_64 but not dasd.""")
        return op

class F28_ClearPart(F21_ClearPart):
    def __init__(self, *args, **kwargs):
        super(F28_ClearPart, self).__init__(*args, **kwargs)
        self.cdl = kwargs.get("cdl", False)

    def __str__(self):
        s = super(F28_ClearPart, self).__str__()
        if s and self.cdl:
            s = s.rstrip()
            s += " --cdl\n"
        return s

    def _getParser(self):
        op = super(F28_ClearPart, self)._getParser()
        op.add_argument("--cdl", dest="cdl", default=False, version=F28,
                        action="store_true", help="""
                        Reformat any LDL DASDs to CDL format.""")
        return op

class RHEL7_ClearPart(F28_ClearPart):
    pass



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, F8
from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

import six

class FC3_RootPw(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.isCrypted = kwargs.get("isCrypted", False)
        self.password = kwargs.get("password", "")

    def _getArgsAsStr(self):
        retval = ""

        if self.isCrypted:
            retval += " --iscrypted"

        return retval

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.password:
            if isinstance(self.password, six.binary_type) and b'#' in self.password:
                password = '\"' + self.password + '\"'
            elif not isinstance(self.password, six.binary_type) and u'#' in self.password:
                password = '\"' + self.password + '\"'
            else:
                password = self.password

            retval += "# Root password\nrootpw%s %s\n" % (self._getArgsAsStr(), password)

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="rootpw", description="""
                            This required command sets the system's root
                            password.""", version=FC3)
        op.add_argument('password', metavar='<password>', nargs='?', version=FC3,
                        help="The desired root password.")
        op.add_argument("--iscrypted", dest="isCrypted", action="store_true",
                        default=False, version=FC3, help="""
                        If this is present, the password argument is assumed to
                        already be encrypted. To create an encrypted password
                        you can use python::

                        ``python -c 'import crypt; print(crypt.crypt("My Password", "$6$My Salt"))'``

                        This will generate sha512 crypt of your password using
                        your provided salt.""")
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if not ns.password:
            raise KickstartParseError(_("A single argument is expected for the %s command") % "rootpw", lineno=self.lineno)
        elif extra:
            mapping = {"command": "rootpw", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        self.set_to_self(ns)
        return self

class F8_RootPw(FC3_RootPw):
    removedKeywords = FC3_RootPw.removedKeywords
    removedAttrs = FC3_RootPw.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        FC3_RootPw.__init__(self, writePriority, *args, **kwargs)
        self.lock = kwargs.get("lock", False)

    def _getArgsAsStr(self):
        retval = FC3_RootPw._getArgsAsStr(self)

        if self.lock:
            retval += " --lock"

        if not self.isCrypted:
            retval += " --plaintext"

        return retval

    def _getParser(self):
        op = FC3_RootPw._getParser(self)
        op.add_argument("--lock", action="store_true", default=False,
                        version=F8, help="""
                        If this is present, the root account is locked by
                        default. That is, the root user will not be able to
                        login from the console. When this option is present
                        the <password> argument is not required.""")
        op.add_argument("--plaintext", dest="isCrypted", action="store_false",
                        version=F8, help="""
                        The password argument is assumed to not be encrypted.
                        This is the default!""")
        return op

class F18_RootPw(F8_RootPw):
    removedKeywords = F8_RootPw.removedKeywords
    removedAttrs = F8_RootPw.removedAttrs

    def __str__(self):
        retval = F8_RootPw.__str__(self)

        if not retval and self.lock:
            retval = "#Root password\nrootpw --lock\n"

        return retval

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if not (ns.password or ns.lock):
            raise KickstartParseError(_("A single argument is expected for the %s command") % "rootpw", lineno=self.lineno)
        elif extra:
            mapping = {"command": "rootpw", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        self.set_to_self(ns)
        return self



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest

from pykickstart.errors import KickstartParseWarning
from tests.baseclass import CommandTest, CommandSequenceTest
from pykickstart.version import F12
from pykickstart.commands.zfcp import FC3_ZFCP, FC3_ZFCPData, F12_ZFCPData

class ZFCP_TestCase(unittest.TestCase):
    def runTest(self):
        cmd = FC3_ZFCP()

        # test option parser
        op = cmd._getParser()
        for action in op._actions:
            for a in ['--devnum', '--fcplun', '--scsiid', '--scsilun', '--wwpn']:
                if a in action.option_strings:
                    self.assertEqual(action.required, True)

        # test that new objects are always equal
        data1 = FC3_ZFCPData()
        data2 = FC3_ZFCPData()
        self.assertEqual(data1, data2)
        self.assertNotEqual(data1, None)

        # additional asserts for code coverage
        self.assertEqual(data1.__str__(), "zfcp\n")
        cmd.zfcp = [data1]
        self.assertEqual(cmd.__str__(), "zfcp\n")

        # test for objects difference
        for atr in ['devnum', 'wwpn', 'fcplun', 'scsiid', 'scsilun']:
            setattr(data1, atr, 1)
            setattr(data2, atr, 2)
            # objects that differ in only one attribute
            # are not equal
            self.assertNotEqual(data1, data2)
            self.assertNotEqual(data2, data1)
            setattr(data1, atr, 0)
            setattr(data2, atr, 0)

        # repeat test for newer data class
        # b/c it uses fewer attributes to compare for equality
        data3 = F12_ZFCPData()
        data4 = F12_ZFCPData()
        self.assertEqual(data3, data4)

        for atr in ['devnum', 'wwpn', 'fcplun']:
            setattr(data3, atr, 3)
            setattr(data4, atr, 4)
            self.assertNotEqual(data3, data4)
            self.assertNotEqual(data4, data3)
            setattr(data3, atr, 0)
            setattr(data4, atr, 0)


class FC3_TestCase(CommandTest):
    def runTest(self):
        # pass
        self.assert_parse("zfcp --devnum=1 --wwpn=2 --fcplun=3 --scsiid=4 --scsilun=5",
                          "zfcp --devnum=1 --wwpn=2 --fcplun=3 --scsiid=4 --scsilun=5\n")

        self.assertFalse(self.assert_parse("zfcp --devnum=1 --wwpn=2 --fcplun=3 --scsiid=4 --scsilun=5") is None)
        self.assertTrue(self.assert_parse("zfcp --devnum=1 --wwpn=2 --fcplun=3 --scsiid=4 --scsilun=5") !=
                        self.assert_parse("zfcp --devnum=6 --wwpn=7 --fcplun=8 --scsiid=9 --scsilun=10"))
        self.assertFalse(self.assert_parse("zfcp --devnum=1 --wwpn=2 --fcplun=3 --scsiid=4 --scsilun=5") ==
                         self.assert_parse("zfcp --devnum=6 --wwpn=7 --fcplun=8 --scsiid=9 --scsilun=10"))
        self.assertTrue(self.assert_parse("zfcp --devnum=1 --wwpn=2 --fcplun=3 --scsiid=4 --scsilun=5") ==
                        self.assert_parse("zfcp --devnum=1 --wwpn=2 --fcplun=3 --scsiid=4 --scsilun=5"))

        # fail
        self.assert_parse_error("zfcp --devnum=1 --wwpn=2 --fcplun=3 --scsiid=4")
        self.assert_parse_error("zfcp --devnum=1 --wwpn=2 --fcplun=3 --scsilun=4")
        self.assert_parse_error("zfcp --devnum=1 --wwpn=2 --fcplun=3")
        self.assert_parse_error("zfcp --devnum --wwpn --fcplun --scsiid --scsilun")

class F12_TestCase(FC3_TestCase):
    def runTest(self):
        # pass
        self.assert_parse("zfcp --devnum=1 --wwpn=2 --fcplun=3",
                          "zfcp --devnum=1 --wwpn=2 --fcplun=3\n")

        # deprecated
        self.assert_deprecated("zfcp", "--scsiid")
        self.assert_deprecated("zfcp", "--scsilun")

        # equality
        self.assertEqual(self.assert_parse("zfcp --devnum=1 --wwpn=2 --fcplun=3"), self.assert_parse("zfcp --devnum=1 --wwpn=2 --fcplun=3"))
        self.assertNotEqual(self.assert_parse("zfcp --devnum=1 --wwpn=2 --fcplun=3"), None)
        self.assertNotEqual(self.assert_parse("zfcp --devnum=1 --wwpn=2 --fcplun=3"), self.assert_parse("zfcp --devnum=10 --wwpn=2 --fcplun=3"))
        self.assertNotEqual(self.assert_parse("zfcp --devnum=1 --wwpn=2 --fcplun=3"), self.assert_parse("zfcp --devnum=1 --wwpn=20 --fcplun=3"))
        self.assertNotEqual(self.assert_parse("zfcp --devnum=1 --wwpn=2 --fcplun=3"), self.assert_parse("zfcp --devnum=1 --wwpn=2 --fcplun=30"))

class F12_Duplicate_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = F12

    def runTest(self):
        self.assert_parse("""
zfcp --devnum=1 --wwpn=2 --fcplun=3
zfcp --devnum=10 --wwpn=20 --fcplun=30""")

        self.assert_parse_error("""
zfcp --devnum=1 --wwpn=2 --fcplun=3
zfcp --devnum=1 --wwpn=2 --fcplun=3""", KickstartParseWarning)

class F14_TestCase(F12_TestCase):
    def runTest(self):
        F12_TestCase.runTest(self)

        self.assert_removed("zfcp", "--scsiid")
        self.assert_removed("zfcp", "--scsilun")

class RHEL8_TestCase(F14_TestCase):
    def runTest(self):
        F14_TestCase.runTest(self)

        # pass
        self.assert_parse("zfcp --devnum=1", "zfcp --devnum=1\n")

        # fail
        self.assert_parse_error("zfcp --wwpn=2 --fcplun=3")
        self.assert_parse_error("zfcp --devnum=1 --wwpn=2")
        self.assert_parse_error("zfcp --devnum=1 --fcplun=3")

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, F26
from pykickstart.base import KickstartCommand
from pykickstart.constants import DISPLAY_MODE_CMDLINE, DISPLAY_MODE_GRAPHICAL, \
                                  DISPLAY_MODE_TEXT
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class FC3_DisplayMode(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.displayMode = kwargs.get("displayMode", None)
        self.op = self._getParser()

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.displayMode is None:
            return retval

        if self.displayMode == DISPLAY_MODE_CMDLINE:
            retval += "cmdline\n"
        elif self.displayMode == DISPLAY_MODE_GRAPHICAL:
            retval += "# Use graphical install\ngraphical\n"
        elif self.displayMode == DISPLAY_MODE_TEXT:
            retval += "# Use text mode install\ntext\n"

        return retval

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)

        if self.currentCmd == "cmdline":
            self.displayMode = DISPLAY_MODE_CMDLINE
        elif self.currentCmd == "graphical":
            self.displayMode = DISPLAY_MODE_GRAPHICAL
        elif self.currentCmd == "text":
            self.displayMode = DISPLAY_MODE_TEXT
        else:
            raise KickstartParseError(_("Unknown command %s") % self.currentCmd, lineno=self.lineno)

        return self

    def _getParser(self):
        op = KSOptionParser(prog="graphical|text|cmdline", version=FC3,
                            description="""
                            Controls which display mode will be used during
                            installation. If ``cmdline`` is chosen all required
                            installation options must be configured via kickstart
                            otherwise the installation will fail.""")
        return op

class F26_DisplayMode(FC3_DisplayMode):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        FC3_DisplayMode.__init__(self, writePriority, args, kwargs)
        self.nonInteractive = kwargs.get("nonInteractive", False)

    def __str__(self):
        retval = super(F26_DisplayMode, self).__str__()

        if self.nonInteractive:
            retval = retval.rstrip()
            retval += " --non-interactive\n"

        return retval

    def _getParser(self):
        op = FC3_DisplayMode._getParser(self)
        op.add_argument("--non-interactive", action="store_true", default=False,
                      dest="nonInteractive", version=F26, help="""
                       Perform the installation in a completely non-interactive mode.
                       This mode will kill the installation when user interaction will be
                       required. Can't be used with ``cmdline`` mode. This option is
                       especially useful for automated testing purpose.""")
        return op

    def parse(self, args):
        FC3_DisplayMode.parse(self, args)

        if self.currentCmd == "cmdline" and self.nonInteractive:
            msg = _("Kickstart command cmdline does not support --non-interactive parameter")
            raise KickstartParseError(msg, lineno=self.lineno)

        return self



#
# Chris Lumens <clumens@redhat.com>
# David Lehman <dlehman@redhat.com>
#
# Copyright 2005, 2006, 2007, 2011 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import F17, F23, RHEL8, versionToLongString
from pykickstart.base import BaseData, KickstartCommand, DeprecatedCommand
from pykickstart.errors import KickstartParseError, KickstartParseWarning
from pykickstart.options import KSOptionParser

import warnings
from pykickstart.i18n import _

class F17_BTRFSData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.format = kwargs.get("format", True)
        self.preexist = kwargs.get("preexist", False)
        self.label = kwargs.get("label", "")
        self.mountpoint = kwargs.get("mountpoint", "")
        self.devices = kwargs.get("devices", [])
        self.dataLevel = kwargs.get("data", None) or kwargs.get("dataLevel", None)
        self.metaDataLevel = kwargs.get("metadata", None) or kwargs.get("metaDataLevel", None)

        # subvolume-specific
        self.subvol = kwargs.get("subvol", False)
        self.parent = kwargs.get("parent", "")
        self.name = kwargs.get("name", None)        # required

    def __eq__(self, y):
        if not y:
            return False

        return self.mountpoint == y.mountpoint

    def __ne__(self, y):
        return not self == y

    def _getArgsAsStr(self):
        retval = ""
        if not self.format:
            retval += " --noformat"
        if self.preexist:
            retval += " --useexisting"
        if self.label:
            retval += " --label=%s" % self.label
        if self.dataLevel:
            retval += " --data=%s" % self.dataLevel.lower()
        if self.metaDataLevel:
            retval += " --metadata=%s" % self.metaDataLevel.lower()
        if self.subvol:
            retval += " --subvol --name=%s" % self.name

        return retval

    def __str__(self):
        retval = BaseData.__str__(self)
        retval += "btrfs %s" % self.mountpoint
        retval += self._getArgsAsStr()
        return retval + " " + " ".join(self.devices) + "\n"

class F23_BTRFSData(F17_BTRFSData):
    removedKeywords = F17_BTRFSData.removedKeywords
    removedAttrs = F17_BTRFSData.removedAttrs

    def __init__(self, *args, **kwargs):
        F17_BTRFSData.__init__(self, *args, **kwargs)
        self.mkfsopts = kwargs.get("mkfsoptions", "") or kwargs.get("mkfsopts", "")

    def _getArgsAsStr(self):
        retval = F17_BTRFSData._getArgsAsStr(self)

        if self.mkfsopts:
            retval += " --mkfsoptions=\"%s\"" % self.mkfsopts

        return retval

class RHEL7_BTRFSData(F23_BTRFSData):
    pass

class F17_BTRFS(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=132, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        # A dict of all the RAID levels we support.  This means that if we
        # support more levels in the future, subclasses don't have to
        # duplicate too much.
        self.levelMap = {"raid0": "raid0", "0": "raid0",
                         "raid1": "raid1", "1": "raid1",
                         "raid10": "raid10", "10": "raid10",
                         "single": "single"}

        self.btrfsList = kwargs.get("btrfsList", [])

    def __str__(self):
        retval = ""
        for btr in self.btrfsList:
            retval += btr.__str__()

        return retval

    def _getParser(self):
        def level_cb(value):
            if value.lower() in self.levelMap:
                return self.levelMap[value.lower()]
            else:
                raise KickstartParseError(_("Invalid btrfs level: %s") % value, lineno=self.lineno)

        op = KSOptionParser(prog="btrfs", description="""
                            Defines a BTRFS volume or subvolume. This command
                            is of the form:

                            ``btrfs <mntpoint> --data=<level> --metadata=<level> --label=<label> <partitions*>``

                            for volumes and of the form:

                            ``btrfs <mntpoint> --subvol --name=<path> <parent>``

                            for subvolumes.

                            The ``<partitions*>`` (which denotes that multiple
                            partitions can be listed) lists the BTRFS identifiers
                            to add to the BTRFS volume. For subvolumes, should be
                            the identifier of the subvolume's parent volume.

                            ``<mntpoint>``

                            Location where the file system is mounted.""",
                            epilog="""
                            The following example shows how to create a BTRFS
                            volume from member partitions on three disks with
                            subvolumes for root and home. The main volume is not
                            mounted or used directly in this example -- only
                            the root and home subvolumes::

                                part btrfs.01 --size=6000 --ondisk=sda
                                part btrfs.02 --size=6000 --ondisk=sdb
                                part btrfs.03 --size=6000 --ondisk=sdc

                                btrfs none --data=0 --metadata=1 --label=f17 btrfs.01 btrfs.02 btrfs.03
                                btrfs / --subvol --name=root LABEL=f17
                                btrfs /home --subvol --name=home f17""",
                            version=F17)
        op.add_argument("--noformat", dest="format", action="store_false",
                        default=True, version=F17, help="""
                        Use an existing BTRFS volume (or subvolume) and do not
                        reformat the filesystem.""")
        op.add_argument("--useexisting", dest="preexist", action="store_true",
                        default=False, help="Same as ``--noformat``.",
                        version=F17)

        # label, data, metadata
        op.add_argument("--label", default="", version=F17, help="""
                        Specify the label to give to the filesystem to be made.
                        If the given label is already in use by another
                        filesystem, a new label will be created. This option
                        has no meaning for subvolumes.""")
        op.add_argument("--data", dest="dataLevel", type=level_cb, help="""
                        RAID level to use (0, 1, 10) for filesystem data. Optional.
                        This option has no meaning for subvolumes.""",
                        version=F17)
        op.add_argument("--metadata", dest="metaDataLevel", type=level_cb,
                        version=F17, help="""
                        RAID level to use (0, 1, 10) for filesystem/volume
                        metadata. Optional. This option has no meaning for
                        subvolumes.""")

        #
        # subvolumes
        #
        op.add_argument("--subvol", action="store_true", default=False,
                        version=F17, help="Create BTRFS subvolume.")

        # parent must be a device spec (LABEL, UUID, &c)
        op.add_argument("--parent", default="", version=F17, help="")
        op.add_argument("--name", default="", version=F17, help="""
                        Subvolume name.""")
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        data = self.dataClass()  # pylint: disable=not-callable
        self.set_to_obj(ns, data)
        data.lineno = self.lineno

        if not data.format:
            data.preexist = True
        elif data.preexist:
            data.format = False

        if not extra:
            raise KickstartParseError(_("btrfs must be given a mountpoint"), lineno=self.lineno)
        elif any(arg for arg in extra if arg.startswith("-")):
            mapping = {"command": "btrfs", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        data.mountpoint = extra[0]
        data.devices = extra[1:]

        if not any([data.devices, data.subvol]):
            raise KickstartParseError(_("btrfs must be given a list of partitions"), lineno=self.lineno)
        elif not data.devices:
            raise KickstartParseError(_("btrfs subvol requires specification of parent volume"), lineno=self.lineno)

        if data.subvol and not data.name:
            raise KickstartParseError(_("btrfs subvolume requires a name"), lineno=self.lineno)

        # Check for duplicates in the data list.
        if data in self.dataList():
            warnings.warn(_("A btrfs volume with the mountpoint %s has already been defined.") % data.label, KickstartParseWarning)

        return data

    def dataList(self):
        return self.btrfsList

    @property
    def dataClass(self):
        return self.handler.BTRFSData

class F23_BTRFS(F17_BTRFS):
    removedKeywords = F17_BTRFS.removedKeywords
    removedAttrs = F17_BTRFS.removedAttrs

    def _getParser(self):
        op = F17_BTRFS._getParser(self)
        op.add_argument("--mkfsoptions", dest="mkfsopts", version=F23, help="""
                        Specifies additional parameters to be passed to the
                        program that makes a filesystem on this partition. No
                        processing is done on the list of arguments, so they
                        must be supplied in a format that can be passed directly
                        to the mkfs program. This means multiple options should
                        be comma-separated or surrounded by double quotes,
                        depending on the filesystem.""")
        return op

    def parse(self, args):
        data = F17_BTRFS.parse(self, args)

        if (data.preexist or not data.format) and data.mkfsopts:
            raise KickstartParseError(_("--mkfsoptions with --noformat or --useexisting has no effect."), lineno=self.lineno)

        return data

class RHEL7_BTRFS(F23_BTRFS):
    pass

class RHEL8_BTRFS(DeprecatedCommand, F23_BTRFS):
    def __init__(self):  # pylint: disable=super-init-not-called
        DeprecatedCommand.__init__(self)

    def _getParser(self):
        op = F23_BTRFS._getParser(self)
        op.description += "\n\n.. deprecated:: %s" % versionToLongString(RHEL8)
        return op



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2007, 2009, 2013 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, FC6
from pykickstart.base import KickstartCommand
from pykickstart.options import KSOptionParser

class FC3_NFS(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.server = kwargs.get("server", None)
        self.dir = kwargs.get("dir", None)

        self.op = self._getParser()

    def __eq__(self, other):
        if not other:
            return False

        return self.server == other.server and self.dir == other.dir

    def __ne__(self, other):
        return not self == other

    def __str__(self):
        retval = KickstartCommand.__str__(self)
        if not self.seen:
            return retval

        retval += "# Use NFS installation media\nnfs --server=%s --dir=%s\n" % (self.server, self.dir)
        return retval

    def _getParser(self):
        op = KSOptionParser(prog="nfs", description="""
                            Install from the NFS server specified. This can
                            either be an exploded installation tree or a
                            directory of ISO images. In the latter case, the
                            install.img must also be provided subject to the
                            same rules as with the harddrive installation
                            method described above.""", version=FC3)
        op.add_argument("--server", metavar="<hostname>", required=True,
                        version=FC3, help="""
                        Server from which to install (hostname or IP).""")
        op.add_argument("--dir", metavar="<directory>", required=True,
                        version=FC3, help="""
                        Directory containing the ``Packages/`` directory of the
                        installation tree. If doing an ISO install, this
                        directory must also contain images/install.img.""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        return self

class FC6_NFS(FC3_NFS):
    removedKeywords = FC3_NFS.removedKeywords
    removedAttrs = FC3_NFS.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        FC3_NFS.__init__(self, writePriority, *args, **kwargs)
        self.opts = kwargs.get("opts", None)

    def __eq__(self, other):
        if not FC3_NFS.__eq__(self, other):
            return False

        return self.opts == other.opts

    def __str__(self):
        retval = FC3_NFS.__str__(self)

        if self.seen and self.opts:
            retval = retval.rstrip()
            retval += " --opts=\"%s\"\n" % self.opts

        return retval

    def _getParser(self):
        op = FC3_NFS._getParser(self)
        op.add_argument("--opts", metavar="<options>", version=FC6, help="""
                        Mount options to use for mounting the NFS export. Any
                        options that can be specified in ``/etc/fstab`` for an NFS
                        mount are allowed. The options are listed in the ``nfs(5)``
                        man page. Multiple options are separated with a comma.
                        """)
        return op



#
# Chris Lumens <clumens@redhat.com>
# Peter Jones <pjones@redhat.com>
#
# Copyright 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC6
from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class FC6_IscsiName(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=70, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()
        self.iscsiname = kwargs.get("iscsiname", "")

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.iscsiname:
            retval += "iscsiname %s\n" % self.iscsiname

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="iscsiname", description="""
            Assigns an initiator name to the computer. If you use the iscsi
            parameter in your kickstart file, this parameter is mandatory, and
            you must specify iscsiname in the kickstart file before you specify
            iscsi.""", version=FC6)
        op.add_argument("iqn", metavar="<iqn>", nargs=1, version=FC6, help="""
                        IQN name""")
        return op

    def parse(self, args):
        (_ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)
        assert len(_ns.iqn) == 1

        if extra:
            mapping = {"command": "iscsiname", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        self.iscsiname = _ns.iqn[0]
        return self



#
# Martin Gracik <mgracik@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.liveimg import F19_Liveimg

class LiveImg_TestCase(unittest.TestCase):
    def runTest(self):
        cmd = F19_Liveimg()
        self.assertFalse(cmd == None)

        op = cmd._getParser()
        for action in op._actions:
            if '--url' in action.option_strings:
                self.assertEqual(action.required, True)
            elif '--noverifyssl' in action.option_strings:
                self.assertEqual(action.default, False)

class F19_TestCase(CommandTest):
    def runTest(self):
        # pass
        self.assert_parse("liveimg --url=http://someplace/somewhere --proxy=http://wherever/other "
                          "--noverifyssl --checksum=e7a9fe500330a1cae4ca114833bb3df014e6d14e63ea9566896a848f3832d0ba",
                          "liveimg --url=\"http://someplace/somewhere\" --proxy=\"http://wherever/other\" "
                          "--noverifyssl --checksum=\"e7a9fe500330a1cae4ca114833bb3df014e6d14e63ea9566896a848f3832d0ba\"\n")
        self.assert_parse("liveimg --url=http://someplace/somewhere --proxy=http://wherever/other "
                          "--noverifyssl",
                          "liveimg --url=\"http://someplace/somewhere\" --proxy=\"http://wherever/other\" "
                          "--noverifyssl\n")
        self.assert_parse("liveimg --url=http://someplace/somewhere --proxy=http://wherever/other ",
                          "liveimg --url=\"http://someplace/somewhere\" --proxy=\"http://wherever/other\"\n")
        self.assert_parse("liveimg --url=http://someplace/somewhere",
                          "liveimg --url=\"http://someplace/somewhere\"\n")

        # equality
        self.assertEqual(self.assert_parse("liveimg --url=http://one"), self.assert_parse("liveimg --url=http://one"))
        self.assertEqual(self.assert_parse("liveimg --url=http://one --proxy=http://wherever"), self.assert_parse("liveimg --url=http://one --proxy=http://wherever"))
        self.assertEqual(self.assert_parse("liveimg --url=http://one --noverifyssl"), self.assert_parse("liveimg --url=http://one --noverifyssl"))
        self.assertEqual(self.assert_parse("liveimg --url=http://one --checksum=deadbeef"), self.assert_parse("liveimg --url=http://one --checksum=deadbeef"))

        self.assertNotEqual(self.assert_parse("liveimg --url=http://one"), self.assert_parse("liveimg --url=http://two"))
        self.assertNotEqual(self.assert_parse("liveimg --url=http://one --proxy=http://wherever"), self.assert_parse("liveimg --url=http://two"))
        self.assertNotEqual(self.assert_parse("liveimg --url=http://one --proxy=http://wherever"), self.assert_parse("liveimg --url=http://one, --proxy=http://somewhere"))
        self.assertNotEqual(self.assert_parse("liveimg --url=http://one --noverifyssl"), self.assert_parse("liveimg --url=http://one"))
        self.assertNotEqual(self.assert_parse("liveimg --url=http://one --checksum=deadbeef"), self.assert_parse("liveimg --url=http://one"))
        self.assertNotEqual(self.assert_parse("liveimg --url=http://one --checksum=deadbeef"), self.assert_parse("liveimg --url=http://one --checksum=abababab"))

        self.assertFalse(self.assert_parse("liveimg --url=http://one") == None)

        # fail
        self.assert_parse_error("liveimg")
        self.assert_parse_error("liveimg --url")
        self.assert_parse_error("liveimg --url=http://someplace/somewhere --proxy")
        self.assert_parse_error("liveimg --proxy=http://someplace/somewhere")
        self.assert_parse_error("liveimg --noverifyssl")
        self.assert_parse_error("liveimg --checksum=e7a9fe500330a1cae4ca114833bb3df014e6d14e63ea9566896a848f3832d0ba")

if __name__ == "__main__":
    unittest.main()



#
# Martin Gracik <mgracik@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
import copy
from tests.baseclass import CommandTest

class FC3_Proxy_TestCase(CommandTest):
    command = "method"

    def runTest(self):
        # Test the default method.
        handler = self.handler()
        self._test_seen(handler, None)

        # Try to access correct attribute.
        handler = self.handler()
        handler.method.method = "nfs"

        setattr(handler.method, "server", "1.2.3.4")
        handler.method.server = "1.2.3.4"
        getattr(handler.method, "server")
        if handler.method.server:
            pass

        # Try to access wrong attribute.
        handler = self.handler()
        handler.method.method = "url"

        with self.assertRaises(AttributeError):
            setattr(handler.method, "server", "1.2.3.4")

        with self.assertRaises(AttributeError):
            handler.method.server = "1.2.3.4"

        with self.assertRaises(AttributeError):
            getattr(handler.method, "server")

        with self.assertRaises(AttributeError):
            if handler.method.server:
                pass

        # Try to set a nonexistent method.
        handler = self.handler()
        with self.assertRaises(AttributeError):
            handler.method.method = "???"

        # Try to get and set a nonexistent attribute.
        handler = self.handler()
        handler.method.method = "url"

        with self.assertRaises(AttributeError):
            getattr(handler.method, "nonexistent_attribute")

        with self.assertRaises(AttributeError):
            if handler.method.nonexistent_attribute:
                pass

        with self.assertRaises(AttributeError):
            setattr(handler.method, "nonexistent_attribute", "some_value")

        with self.assertRaises(AttributeError):
            handler.method.nonexistent_attribute = "some_value"

        # Try to call a nonexistent method.
        handler = self.handler()
        with self.assertRaises(AttributeError):
            handler.method.nonexistent_method()

        # Test hasattr.
        handler = self.handler()
        self.assertEqual(hasattr(handler.method, "seen"), True)
        self.assertEqual(hasattr(handler.method, "server"), False)
        self.assertEqual(hasattr(handler.method, "nonexistent"), False)

        handler.method.method = "nfs"
        self.assertEqual(hasattr(handler.method, "seen"), True)
        self.assertEqual(hasattr(handler.method, "server"), True)
        self.assertEqual(hasattr(handler.method, "nonexistent"), False)

        # Test copy and deepcopy.
        handler = self.handler()
        handler.method.method = "url"
        handler.method.url = "http://domain.com"

        method2 = copy.copy(handler.method)
        self.assertEqual(method2.method, "url")
        self.assertEqual(method2.url, "http://domain.com")

        # TODO: Add test for copy.deepcopy.
        # The test now fails somewhere outside of the method command.
        # It should be fixed or anaconda should stop calling it.

        # Test an internal attribute.
        handler = self.handler()
        handler.method.method = "url"
        self.assertEqual(handler.method.lineno, 0)
        self.assertEqual(handler.url.lineno, 0)

        handler.method.lineno = 5
        self.assertEqual(handler.method.lineno, 5)
        self.assertEqual(handler.url.lineno, 0)

        handler.url.lineno = 10
        self.assertEqual(handler.method.lineno, 5)
        self.assertEqual(handler.url.lineno, 10)

        # Test command's attributes.
        # Try to switch between methods.
        handler = self.handler()
        self._set_default(handler)
        self._set_cdrom(handler)
        self._set_url(handler)
        self._set_nfs(handler)
        self._set_harddrive(handler)

    def _test_seen(self, handler, seen_method):
        self.assertEqual(handler.method.method, seen_method)

        for method in handler.method._methods:
            self.assertEqual(getattr(handler, method).seen, method == seen_method)

    def _set_default(self, handler):
        handler.method.method = None
        self._test_seen(handler, None)

        handler.method.url = "http://domain.com"
        self.assertEqual(handler.method.url, "http://domain.com")
        self.assertEqual(handler.url.url, "http://domain.com")
        self.assertEqual(getattr(handler.method, "url"), "http://domain.com")

    def _set_cdrom(self, handler):
        handler.method.method = "cdrom"
        self._test_seen(handler, "cdrom")

    def _set_url(self, handler):
        handler.method.method = "url"
        self._test_seen(handler, "url")

        handler.method.url = "http://domain.com"
        self.assertEqual(handler.method.url, "http://domain.com")
        self.assertEqual(handler.url.url, "http://domain.com")
        self.assertEqual(getattr(handler.method, "url"), "http://domain.com")

    def _set_nfs(self, handler):
        handler.method.method = "nfs"
        self._test_seen(handler, "nfs")

        handler.method.server = "1.2.3.4"
        self.assertEqual(handler.method.server, "1.2.3.4")
        self.assertEqual(handler.nfs.server, "1.2.3.4")
        self.assertEqual(getattr(handler.method, "server"), "1.2.3.4")

    def _set_harddrive(self, handler):
        handler.method.method = "harddrive"
        self._test_seen(handler, "harddrive")

        handler.method.dir = "/install"
        self.assertEqual(handler.method.dir, "/install")
        self.assertEqual(handler.harddrive.dir, "/install")
        self.assertEqual(getattr(handler.method, "dir"), "/install")

class FC3_TestCase(CommandTest):
    command = "method"

    def runTest(self):
        # pass
        # cdrom
        self.assert_parse("cdrom", "cdrom\n")

        # harddrive
        self.assert_parse("harddrive --dir=/install --biospart=part", "harddrive --dir=/install --biospart=part\n")
        self.assert_parse("harddrive --dir=/install --partition=part", "harddrive --dir=/install --partition=part\n")

        # nfs
        self.assert_parse("nfs --server=1.2.3.4 --dir=/install", "nfs --server=1.2.3.4 --dir=/install\n")

        # url
        self.assert_parse("url --url=http://domain.com", "url --url=\"http://domain.com\"\n")

        # fail
        # harddrive
        # required option --dir missing
        self.assert_parse_error("harddrive")
        # required --dir argument missing
        self.assert_parse_error("harddrive --dir")
        # missing --biospart or --partition option
        self.assert_parse_error("harddrive --dir=/install")
        # both --biospart and --partition specified
        self.assert_parse_error("harddrive --dir=/install --biospart=bios --partition=part")
        # --biospart and --partition require argument
        self.assert_parse_error("harddrive --dir=/install --biospart")
        self.assert_parse_error("harddrive --dir=/install --partition")
        # unknown option
        self.assert_parse_error("harddrive --unknown=value")

        # nfs
        # missing required options --server and --dir
        self.assert_parse_error("nfs")
        self.assert_parse_error("nfs --server=1.2.3.4")
        self.assert_parse_error("nfs --server")
        self.assert_parse_error("nfs --dir=/install")
        self.assert_parse_error("nfs --dir")
        # unknown option
        self.assert_parse_error("nfs --unknown=value")

        # url
        # missing required option --url
        self.assert_parse_error("url")
        self.assert_parse_error("url --url")

class FC6_TestCase(FC3_TestCase):
    def runTest(self):
        # run FC3 test case
        FC3_TestCase.runTest(self)

        # pass
        # nfs
        self.assert_parse("nfs --server=1.2.3.4 --dir=/install --opts=options", "nfs --server=1.2.3.4 --dir=/install --opts=\"options\"\n")

        # fail
        # nfs
        # --opts requires argument if specified
        self.assert_parse_error("nfs --server=1.2.3.4 --dir=/install --opts")


class F13_TestCase(FC6_TestCase):
    def runTest(self):
        # run FC6 test case
        FC6_TestCase.runTest(self)

        # pass
        self.assert_parse("url --url=http://someplace/somewhere --proxy=http://wherever/other",
                          "url --url=\"http://someplace/somewhere\" --proxy=\"http://wherever/other\"\n")

        # fail
        self.assert_parse_error("cdrom --proxy=http://someplace/somewhere")
        self.assert_parse_error("url --url=http://someplace/somewhere --proxy")
        self.assert_parse_error("url --proxy=http://someplace/somewhere")

class F14_TestCase(F13_TestCase):
    def runTest(self):
        # run FC6 test case
        F13_TestCase.runTest(self)

        # pass
        self.assert_parse("url --url=https://someplace/somewhere --noverifyssl",
                          "url --url=\"https://someplace/somewhere\" --noverifyssl\n")

        # fail
        self.assert_parse_error("cdrom --noverifyssl")

class F18_TestCase(F14_TestCase):
    def runTest(self):
        # run F14 test case.
        F14_TestCase.runTest(self)

        # pass
        self.assert_parse("url --mirrorlist=http://www.wherever.com/mirror",
                          "url --mirrorlist=\"http://www.wherever.com/mirror\"\n")

        # fail
        # missing one of required options --url or --mirrorlist
        self.assert_parse_error("url")
        self.assert_parse_error("url --mirrorlist")

        # It's --url, not --baseurl.
        self.assert_parse_error("url --baseurl=www.wherever.com")

        # only one of --url or --mirrorlist may be specified
        self.assert_parse_error("url --url=www.wherever.com --mirrorlist=www.wherever.com")

class F19_Proxy_TestCase(FC3_Proxy_TestCase):
    def runTest(self):
        FC3_Proxy_TestCase.runTest(self)

        # Test command's attributes.
        handler = self.handler()
        self._set_liveimg(handler)

        # Try to switch between methods.
        handler = self.handler()
        self._set_url(handler)
        self._set_liveimg(handler)
        self._set_nfs(handler)

    def _set_liveimg(self, handler):
        handler.method.method = "liveimg"
        self._test_seen(handler, "liveimg")

        handler.method.url = "http://someplace/somewhere"
        self.assertEqual(handler.method.url, "http://someplace/somewhere")
        self.assertEqual(handler.liveimg.url, "http://someplace/somewhere")
        self.assertEqual(getattr(handler.liveimg, "url"), "http://someplace/somewhere")

class F19_TestCase(F18_TestCase):
    def runTest(self):
        # run F18 test case.
        F18_TestCase.runTest(self)

        # liveimg pass
        self.assert_parse("liveimg --url=http://someplace/somewhere --proxy=http://wherever/other "
                          "--noverifyssl --checksum=e7a9fe500330a1cae4ca114833bb3df014e6d14e63ea9566896a848f3832d0ba",
                          "liveimg --url=\"http://someplace/somewhere\" --proxy=\"http://wherever/other\" "
                          "--noverifyssl --checksum=\"e7a9fe500330a1cae4ca114833bb3df014e6d14e63ea9566896a848f3832d0ba\"\n")
        self.assert_parse("liveimg --url=http://someplace/somewhere --proxy=http://wherever/other "
                          "--noverifyssl",
                          "liveimg --url=\"http://someplace/somewhere\" --proxy=\"http://wherever/other\" "
                          "--noverifyssl\n")
        self.assert_parse("liveimg --url=http://someplace/somewhere --proxy=http://wherever/other ",
                          "liveimg --url=\"http://someplace/somewhere\" --proxy=\"http://wherever/other\"\n")
        self.assert_parse("liveimg --url=http://someplace/somewhere",
                          "liveimg --url=\"http://someplace/somewhere\"\n")

        # liveimg fail
        self.assert_parse_error("liveimg")
        self.assert_parse_error("liveimg --url")
        self.assert_parse_error("liveimg --url=http://someplace/somewhere --proxy")
        self.assert_parse_error("liveimg --proxy=http://someplace/somewhere")
        self.assert_parse_error("liveimg --noverifyssl")
        self.assert_parse_error("liveimg --checksum=e7a9fe500330a1cae4ca114833bb3df014e6d14e63ea9566896a848f3832d0ba")

class RHEL7_Proxy_TestCase(F19_Proxy_TestCase):
    def runTest(self):
        F19_Proxy_TestCase.runTest(self)

        # Test command's attributes.
        handler = self.handler()
        self._set_hmc(handler)

        # Try to switch between methods.
        handler = self.handler()
        self._set_cdrom(handler)
        self._set_hmc(handler)
        self._set_harddrive(handler)

    def _set_hmc(self, handler):
        handler.method.method = "hmc"
        self._test_seen(handler, "hmc")

class RHEL7_TestCase(F19_TestCase):
    def runTest(self):
        F19_TestCase.runTest(self)

        # Test hmc.
        self.assert_parse("hmc", "hmc\n")

class F28_Proxy_TestCase(RHEL7_Proxy_TestCase):
    def runTest(self):
        RHEL7_Proxy_TestCase.runTest(self)

class F28_TestCase(RHEL7_TestCase):
    def runTest(self):
        RHEL7_TestCase.runTest(self)

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005-2016 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
"""
Specialized option handling.

This module exports three classes:

    ExtendAction - A subclass of Action that appends a list of values to an
                   already existing list.  In this way, it's like the existing
                   "append" action except for lists instead of single values.

    ExtendConstAction - A subclass of Action that appends a list of constants
                        to an already existing list.  In this way, it's like the
                        existing "append_const" action except for lists instead
                        of single values.

    KSOptionParser - A specialized subclass of ArgumentParser to be used
                     in BaseHandler subclasses.

And it exports two functions:

    commaSplit - A function to be used as the type= argument to any arguments
                 that take a single string that may be split on commas, resulting
                 in a list of strings.

    ksboolean - A function to be used as the type= argument to any arguments
                that can take a boolean.
"""
import warnings
import textwrap
from argparse import RawTextHelpFormatter, SUPPRESS
from argparse import Action, ArgumentParser, ArgumentTypeError

from pykickstart.errors import KickstartParseError, KickstartDeprecationWarning
from pykickstart.version import versionToString, versionToLongString

from pykickstart.i18n import _

def commaSplit(value):
    return list(filter(None, [v.strip() for v in value.split(',')]))

def ksboolean(value):
    try:
        if value.lower() in ("on", "yes", "true", "1"):
            return True
        elif value.lower() in ("off", "no", "false", "0"):
            return False
        else:
            raise ArgumentTypeError(_("invalid boolean value: %r") % value)
    except AttributeError:
        raise ArgumentTypeError(_("invalid boolean value: %r") % value)

class KSHelpFormatter(RawTextHelpFormatter):
    """
        Used in generating documentation
    """

    def _format_usage(self, usage, actions, groups, prefix):
        return "::\n\n    %s" % super(KSHelpFormatter,
                                      self)._format_usage(usage,
                                                          actions,
                                                          groups,
                                                          "").strip()

    def _format_action(self, action):
        text = super(KSHelpFormatter, self)._format_action(action)
        parts = text.strip().split('\n')
        new_parts = []
        new_parts.append("\n``%s``\n" % parts[0].strip())
        for p in parts[1:]:
            if p:
                new_parts.append("    %s" % p.lstrip())
        return self._join_parts(new_parts)

    def _join_parts(self, part_strings):
        return '\n'.join([part.rstrip(' ')
                          for part in part_strings
                          if part and part is not SUPPRESS])


class ExtendAction(Action):
    def __call__(self, parser, namespace, values, option_string=None):
        if getattr(namespace, self.dest, None) is not None:
            setattr(namespace, self.dest, getattr(namespace, self.dest) + values)
        else:
            setattr(namespace, self.dest, values)

class ExtendConstAction(Action):
    def __call__(self, parser, namespace, values, option_string=None):
        if getattr(namespace, self.dest, None) is not None:
            setattr(namespace, self.dest, self.const + values)
        else:
            setattr(namespace, self.dest, self.const)

class KSOptionParser(ArgumentParser):
    """A specialized subclass of argparse.ArgumentParser to handle extra option
       attribute checking, work error reporting into the KickstartParseError
       framework, and to turn off the default help.
    """
    def __init__(self, *args, **kwargs):
        """Create a new KSOptionParser instance.  Each KickstartCommand
           subclass should create one instance of KSOptionParser, providing
           at least the lineno attribute.  version is not required.

           Instance attributes:

           version -- The version of the kickstart syntax we are checking
                      against.
        """
        # Overridden to allow for the version kwargs, to skip help option generation,
        # and to resolve conflicts instead of override earlier options.
        int_version = kwargs.pop("version")  # fail fast if no version is specified
        version = versionToLongString(int_version)

        # always document the version
        addVersion = kwargs.pop('addVersion', True)

        # remove leading spaced from description and epilog.
        # fail fast if we forgot to add description
        kwargs['description'] = textwrap.dedent(kwargs.pop("description"))
        if addVersion:
            kwargs['description'] = "\n.. versionadded:: %s\n%s" % (version,
                                                                    kwargs['description'])
        kwargs['epilog'] = textwrap.dedent(kwargs.pop("epilog", ""))

        # fail fast if we forgot to add prog
        kwargs['prog'] = kwargs.pop("prog")

        # remove leading spaced from description and epilog.
        # fail fast if we forgot to add description
        kwargs['description'] = textwrap.dedent(kwargs.pop("description"))
        kwargs['epilog'] = textwrap.dedent(kwargs.pop("epilog", ""))

        # fail fast if we forgot to add prog
        kwargs['prog'] = kwargs.pop("prog")

        ArgumentParser.__init__(self, add_help=False, conflict_handler="resolve",
                                formatter_class=KSHelpFormatter, *args, **kwargs)
        # NOTE: On Python 2.7 ArgumentParser has a deprecated version parameter
        # which always defaults to self.version = None which breaks deprecation
        # warnings in pykickstart. That's why we always set this value after
        # ArgumentParser.__init__ has been executed
        self.version = int_version
        self.lineno = None

    def _parse_optional(self, arg_string):
        def usedTooNew(action):
            return action.introduced and action.introduced > self.version

        def usedRemoved(action):
            return action.removed and action.removed <= self.version

        option_tuple = ArgumentParser._parse_optional(self, arg_string)
        if option_tuple is None or option_tuple[0] is None:
            return option_tuple

        action = option_tuple[0]

        if usedTooNew(action):
            mapping = {"option": action.option_strings[0], "intro": versionToString(action.introduced),
                       "version": versionToString(self.version)}
            self.error(_("The %(option)s option was introduced in version %(intro)s, but you are using kickstart syntax version %(version)s.") % mapping)
        elif usedRemoved(action):
            mapping = {"option": action.option_strings[0], "removed": versionToString(action.removed),
                       "version": versionToString(self.version)}

            if action.removed == self.version:
                self.error(_("The %(option)s option is no longer supported.") % mapping)
            else:
                self.error(_("The %(option)s option was removed in version %(removed)s, but you are using kickstart syntax version %(version)s.") % mapping)
        elif action.deprecated is True or (self.version and type(action.deprecated) == int and self.version >= action.deprecated):
            mapping = {"lineno": self.lineno, "option": action.option_strings[0]}
            warnings.warn(_("Ignoring deprecated option on line %(lineno)s:  The %(option)s option has been deprecated and no longer has any effect.  It may be removed from future releases, which will result in a fatal error from kickstart.  Please modify your kickstart file to remove this option.") % mapping, KickstartDeprecationWarning)

        return option_tuple

    def add_argument(self, *args, **kwargs):
        introduced = kwargs.pop("introduced", None)
        deprecated = kwargs.pop("deprecated", False)
        if deprecated:
            version = versionToLongString(deprecated)
        else:
            # fail fast if version is missing
            version = versionToLongString(introduced or kwargs.pop("version"))

        candidate = None
        for action in self._actions:
            for arg in args:
                if arg in action.option_strings:
                    candidate = action
                    break

        if candidate:
            if deprecated:
                _help = candidate.help or ""
                _help += "\n\n    .. deprecated:: %s" % version
                kwargs["help"] = _help
            else:
                # this is a modified argument, which is already present
                _help = candidate.help or ""
                _help += "\n\n    .. versionchanged:: %s\n\n%s" % (version, kwargs.pop("help"))
                kwargs["help"] = _help
        else:
            # this is a new argument which is added for the first time
            _help = kwargs.pop("help")
            _help += "\n\n    .. versionadded:: %s" % version
            # there are some argumets which are deprecated on first declaration
            if deprecated:
                _help += "\n\n    .. deprecated:: %s" % version
            kwargs["help"] = _help

        notest = kwargs.pop("notest", False)
        removed = kwargs.pop("removed", None)

        action = ArgumentParser.add_argument(self, *args, **kwargs)
        action.deprecated = deprecated
        action.introduced = introduced
        action.notest = notest
        action.removed = removed
        return action

    def remove_argument(self, arg, **kwargs):
        candidate = None

        for action in self._actions:
            if arg in action.option_strings:
                candidate = action
                break

        if candidate:
            if not candidate.help:
                candidate.help = ""
            candidate.help += "\n\n    .. versionremoved:: %s" % versionToLongString(kwargs.pop("version"))
            self._remove_action(candidate)
            self._option_string_actions.pop(arg)

    def error(self, message):
        # Overridden to turn errors into KickstartParseErrors.
        if self.lineno is not None:
            raise KickstartParseError(message, lineno=self.lineno)
        else:
            raise KickstartParseError(message)

    def exit(self, status=0, message=None):
        # Overridden because this is a library, and libraries shouldn't just
        # exit.  That's what raising exceptions is for.
        pass

    def parse_args(self, *args, **kwargs):  # pylint: disable=arguments-differ,signature-differs
        if "lineno" in kwargs:
            self.lineno = kwargs.pop("lineno")

        return ArgumentParser.parse_args(self, *args, **kwargs)

    def parse_known_args(self, *args, **kwargs):  # pylint: disable=arguments-differ,signature-differs
        if "lineno" in kwargs:
            self.lineno = kwargs.pop("lineno")

        return ArgumentParser.parse_known_args(self, *args, **kwargs)



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007, 2008 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, FC6, F9, F10, F14
from pykickstart.base import KickstartCommand
from pykickstart.options import KSOptionParser

class FC3_XConfig(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.card = kwargs.get("card", "")
        self.defaultdesktop = kwargs.get("defaultdesktop", "")
        self.depth = kwargs.get("depth", 0)
        self.hsync = kwargs.get("hsync", "")
        self.monitor = kwargs.get("monitor", "")
        self.noProbe = kwargs.get("noProbe", False)
        self.resolution = kwargs.get("resolution", "")
        self.server = kwargs.get("server", "")
        self.startX = kwargs.get("startX", False)
        self.videoRam = kwargs.get("videoRam", "")
        self.vsync = kwargs.get("vsync", "")

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.card:
            retval += " --card=%s" % self.card
        if self.defaultdesktop:
            retval += " --defaultdesktop=%s" % self.defaultdesktop
        if self.depth != 0:
            retval += " --depth=%d" % self.depth
        if self.hsync:
            retval += " --hsync=%s" % self.hsync
        if self.monitor:
            retval += " --monitor=%s" % self.monitor
        if self.noProbe:
            retval += " --noprobe"
        if self.resolution:
            retval += " --resolution=%s" % self.resolution
        if self.server:
            retval += " --server=%s" % self.server
        if self.startX:
            retval += " --startxonboot"
        if self.videoRam:
            retval += " --videoram=%s" % self.videoRam
        if self.vsync:
            retval += " --vsync=%s" % self.vsync

        if retval:
            retval = "# X Window System configuration information\nxconfig %s\n" % retval

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="xconfig", description="""
                            Configures the X Window System. If this option is
                            not given, Anaconda will use X and attempt to
                            automatically configure. Please try this before
                            manually configuring your system.""", version=FC3)
        op.add_argument("--card", version=FC3, help="")
        op.add_argument("--defaultdesktop", metavar="GNOME|KDE", help="""
                        Specify either GNOME or KDE to set the default desktop
                        (assumes that GNOME Desktop Environment and/or KDE
                        Desktop Environment has been installed through
                        %%packages).""", version=FC3)
        op.add_argument("--depth", type=int, version=FC3, help="")
        op.add_argument("--hsync", version=FC3, help="")
        op.add_argument("--monitor", version=FC3, help="")
        op.add_argument("--noprobe", dest="noProbe", action="store_true",
                        default=False, version=FC3, help="")
        op.add_argument("--resolution", version=FC3, help="")
        op.add_argument("--server", version=FC3, help="")
        op.add_argument("--startxonboot", dest="startX", action="store_true",
                        default=False, version=FC3, help="""
                        Use a graphical login on the installed system.""")
        op.add_argument("--videoram", dest="videoRam", version=FC3, help="")
        op.add_argument("--vsync", version=FC3, help="")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        return self

class FC6_XConfig(FC3_XConfig):
    removedKeywords = FC3_XConfig.removedKeywords + ["card", "hsync", "monitor", "noProbe", "server", "vsync"]
    removedAttrs = FC3_XConfig.removedAttrs + ["card", "hsync", "monitor", "noProbe", "server", "vsync"]

    def __init__(self, writePriority=0, *args, **kwargs):
        FC3_XConfig.__init__(self, writePriority, *args, **kwargs)
        self.deleteRemovedAttrs()

        self.driver = kwargs.get("driver", "")

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if hasattr(self, "driver") and self.driver:
            retval += " --driver=%s" % self.driver
        if self.defaultdesktop:
            retval += " --defaultdesktop=%s" % self.defaultdesktop
        if self.depth != 0:
            retval += " --depth=%d" % self.depth
        if hasattr(self, "resolution") and self.resolution:
            retval += " --resolution=%s" % self.resolution
        if self.startX:
            retval += " --startxonboot"
        if hasattr(self, "videoRam") and self.videoRam:
            retval += " --videoram=%s" % self.videoRam

        if retval:
            retval = "# X Window System configuration information\nxconfig %s\n" % retval

        return retval

    def _getParser(self):
        op = FC3_XConfig._getParser(self)
        op.add_argument("--card", deprecated=FC6)
        op.add_argument("--driver", version=FC6, help="")
        op.add_argument("--hsync", deprecated=FC6)
        op.add_argument("--monitor", deprecated=FC6)
        op.add_argument("--noprobe", deprecated=FC6)
        op.add_argument("--vsync", deprecated=FC6)
        op.remove_argument("--server", version=FC6, help="")
        return op

class F9_XConfig(FC6_XConfig):
    removedKeywords = FC6_XConfig.removedKeywords
    removedAttrs = FC6_XConfig.removedAttrs

    def _getParser(self):
        op = FC6_XConfig._getParser(self)
        op.remove_argument("--card", version=F9)
        op.remove_argument("--hsync", version=F9)
        op.remove_argument("--monitor", version=F9)
        op.remove_argument("--noprobe", version=F9)
        op.remove_argument("--vsync", version=F9)
        return op

class F10_XConfig(F9_XConfig):
    removedKeywords = F9_XConfig.removedKeywords + ["driver", "resolution", "videoRam"]
    removedAttrs = F9_XConfig.removedAttrs + ["driver", "resolution", "videoRam"]

    def __init__(self, writePriority=0, *args, **kwargs):
        F9_XConfig.__init__(self, writePriority, *args, **kwargs)
        self.deleteRemovedAttrs()

    def _getParser(self):
        op = F9_XConfig._getParser(self)
        op.add_argument("--driver", deprecated=F10)
        op.add_argument("--depth", deprecated=F10)
        op.add_argument("--resolution", deprecated=F10)
        op.add_argument("--videoram", deprecated=F10)
        return op

class F14_XConfig(F10_XConfig):
    removedKeywords = F10_XConfig.removedKeywords
    removedAttrs = F10_XConfig.removedAttrs

    def _getParser(self):
        op = F10_XConfig._getParser(self)
        op.remove_argument("--driver", version=F14)
        op.remove_argument("--depth", version=F14)
        op.remove_argument("--resolution", version=F14)
        op.remove_argument("--videoram", version=F14)
        return op



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007, 2008 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, FC6, F10, versionToLongString
from pykickstart.base import DeprecatedCommand, KickstartCommand
from pykickstart.options import KSOptionParser

class FC3_Monitor(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.hsync = kwargs.get("hsync", "")
        self.monitor = kwargs.get("monitor", "")
        self.vsync = kwargs.get("vsync", "")

    def __str__(self):
        retval = KickstartCommand.__str__(self)
        retval += "monitor"

        if self.hsync:
            retval += " --hsync=%s" % self.hsync
        if self.monitor:
            retval += " --monitor=\"%s\"" % self.monitor
        if self.vsync:
            retval += " --vsync=%s" % self.vsync

        if retval != "monitor":
            return retval + "\n"
        else:
            return ""

    def _getParser(self):
        op = KSOptionParser(prog="monitor", description="""
                            If the monitor command is not given, anaconda will
                            use X to automatically detect your monitor settings.
                            Please try this before manually configuring your
                            monitor.""", version=FC3)
        op.add_argument("--hsync", version=FC3, help="""
                        Specifies the horizontal sync frequency of the monitor.
                        """)
        op.add_argument("--monitor", version=FC3, help="""
                        Use specified monitor; monitor name should be from the
                        list of monitors in ``/usr/share/hwdata/MonitorsDB`` from
                        the hwdata package. The list of monitors can also be
                        found on the X Configuration screen of the
                        Kickstart Configurator. This is ignored if ``--hsync`` or
                        ``--vsync`` is provided. If no monitor information is
                        provided, the installation program tries to probe for
                        it automatically.""")
        op.add_argument("--vsync", version=FC3, help="""
                        Specifies the vertical sync frequency of the monitor.
                        """)
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        return self

class FC6_Monitor(FC3_Monitor):
    removedKeywords = FC3_Monitor.removedKeywords
    removedAttrs = FC3_Monitor.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        FC3_Monitor.__init__(self, writePriority, *args, **kwargs)
        self.probe = kwargs.get("probe", True)

    def __str__(self):
        retval = KickstartCommand.__str__(self)
        retval += "monitor"

        if self.hsync:
            retval += " --hsync=%s" % self.hsync
        if self.monitor:
            retval += " --monitor=\"%s\"" % self.monitor
        if not self.probe:
            retval += " --noprobe"
        if self.vsync:
            retval += " --vsync=%s" % self.vsync

        if retval != "monitor":
            return retval + "\n"
        else:
            return ""

    def _getParser(self):
        op = FC3_Monitor._getParser(self)
        op.add_argument("--noprobe", dest="probe", action="store_false",
                        default=True, version=FC6, help="""
                        Do not probe the monitor.""")
        return op

class F10_Monitor(DeprecatedCommand, FC6_Monitor):
    def __init__(self):  # pylint: disable=super-init-not-called
        DeprecatedCommand.__init__(self)

    def _getParser(self):
        op = FC6_Monitor._getParser(self)
        op.description += "\n\n.. deprecated:: %s" % versionToLongString(F10)
        return op



#
# Stef Walter <stefw@redhat.com>
#
# Copyright 2013 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import F19
from pykickstart.base import KickstartCommand
from pykickstart.options import KSOptionParser
from pykickstart.errors import KickstartParseError

import getopt
import pipes
import shlex

from pykickstart.i18n import _

class F19_Realm(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.join_realm = None
        self.join_args = []
        self.discover_options = []

    def _parseArguments(self, string):
        if self.join_realm:
            raise KickstartParseError(_("The realm command 'join' should only be specified once"), lineno=self.lineno)
        args = shlex.split(string)
        if not args:
            raise KickstartParseError(_("Missing realm command arguments"), lineno=self.lineno)
        command = args.pop(0)
        if command == "join":
            self._parseJoin(args)
        else:
            raise KickstartParseError(_("Unsupported realm '%s' command") % command, lineno=self.lineno)

    def _parseJoin(self, args):
        try:
            # We only support these args
            opts, remaining = getopt.getopt(args, "", ("client-software=",
                                                       "server-software=",
                                                       "membership-software=",
                                                       "one-time-password=",
                                                       "no-password",
                                                       "computer-ou="))
        except getopt.GetoptError as ex:
            raise KickstartParseError(_("Invalid realm arguments: %s") % ex, lineno=self.lineno)

        if len(remaining) != 1:
            raise KickstartParseError(_("Specify only one realm to join"), lineno=self.lineno)

        # Parse successful, just use this as the join command
        self.join_realm = remaining[0]
        self.join_args = args

        # Build a discovery command
        self.discover_options = []
        supported_discover_options = ("--client-software",
                                      "--server-software",
                                      "--membership-software")
        for (o, a) in opts:
            if o in supported_discover_options:
                self.discover_options.append("%s=%s" % (o, a))

    def _getCommandsAsStrings(self):
        commands = []
        if self.join_args:
            args = [pipes.quote(arg) for arg in self.join_args]
            commands.append("realm join " + " ".join(args))
        return commands

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        commands = self._getCommandsAsStrings()
        if commands:
            retval += "# Realm or domain membership\n"
            retval += "\n".join(commands)
            retval += "\n"

        return retval

    def parse(self, args):
        self._parseArguments(self.currentLine[len(self.currentCmd):].strip())
        return self

    def _getParser(self):
        return KSOptionParser(prog="realm", description="", version=F19)



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2007, 2009, 2013 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3
from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class FC3_HardDrive(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.biospart = kwargs.get("biospart", None)
        self.partition = kwargs.get("partition", None)
        self.dir = kwargs.get("dir", None)

        self.op = self._getParser()

    def __eq__(self, other):
        if not other:
            return False

        return self.biospart == other.biospart and self.partition == other.partition and self.dir == other.dir

    def __ne__(self, other):
        return not self == other

    def __str__(self):
        retval = KickstartCommand.__str__(self)
        if not self.seen:
            return retval

        retval += "# Use hard drive installation media\n"

        if self.biospart is not None:
            retval += "harddrive --dir=%s --biospart=%s\n" % (self.dir, self.biospart)
        else:
            retval += "harddrive --dir=%s --partition=%s\n" % (self.dir, self.partition)

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="harddrive", description="""
            Install from a directory of ISO images on a local drive, which must
            be either vfat or ext2. In addition to this directory, you must also
            provide the install.img in some way. You can either do this by
            booting off the boot.iso or by creating an images/ directory in the
            same directory as the ISO images and placing install.img in there.
            """, version=FC3)
        op.add_argument("--biospart", version=FC3,
                        help="BIOS partition to install from (such as 82p2).")
        op.add_argument("--partition", version=FC3,
                        help="Partition to install from (such as, sdb2).")
        op.add_argument("--dir", required=True, version=FC3, help="""
                        Directory containing both the ISO images and the
                        images/install.img. For example::

                        ``harddrive --partition=hdb2 --dir=/tmp/install-tree``
                        """)
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)

        if self.biospart is None and self.partition is None or \
           self.biospart is not None and self.partition is not None:
            raise KickstartParseError(_("One of biospart or partition options must be specified."), lineno=self.lineno)

        return self
